@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -56,7 +56,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: alert.c,v 1.43 2005/08/14 08:44:48 martinp Exp $
+ *  @version \$Id: alert.c,v 1.52 2006/01/01 22:44:01 martinp Exp $
  *  @file
  */
 
@@ -66,7 +66,7 @@
 
 static void copy_mail(Mail_T, Mail_T);
 static void replace_bare_linefeed(Mail_T *);
-static void substitute(Mail_T*, char *, const char *, const char *);
+static void substitute(Mail_T *, Event_T);
 
 
 /* ------------------------------------------------------------------ Public */
@@ -75,14 +75,21 @@ static void substitute(Mail_T*, char *, const char *, const char *);
 /**
  * Notify registred users about the event
  * @param E An Event object
+ * @return If failed, return HANDLER_ALERT flag or HANDLER_PASSED if passed
  */
-void handle_alert(Event_T E) {
+int handle_alert(Event_T E) {
 
   Service_T s;
+  int rv = HANDLER_PASSED;
 
   ASSERT(E);
 
   s= Event_get_source(E);
+  if(!s) {
+    log("Aborting alert\n");
+    return rv;
+  }
+
   if(s->maillist || Run.maillist) {
     Mail_T m;
     Mail_T n;
@@ -99,8 +106,8 @@ void handle_alert(Event_T E) {
         /* particular event notification type is allowed for given recipient */
         IS_EVENT_SET(m->events, Event_get_id(E)) &&
         (
-          /* 'changed' event type is sent always */
-          E->id == EVENT_CHANGED ||
+          /* 'changed/matched' event types are sent always */
+          E->id == EVENT_CHANGED || E->id == EVENT_MATCH ||
           /* state change notification is sent always */
           E->state_changed       ||
           /* in the case that the state is failed for more cycles we check
@@ -115,22 +122,13 @@ void handle_alert(Event_T E) {
         Mail_T tmp= NULL;
 
         NEW(tmp);
-
         copy_mail(tmp, m);
-
-        if(Event_get_message(E)) {
-          tmp->opt_message= xstrdup(Event_get_message(E));
-        }
-
-        substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
-          Event_get_action_description(E));
-
+        substitute(&tmp, E);
         replace_bare_linefeed(&tmp);
-
         tmp->next= list;
         list= tmp;
 
-        DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
+        DEBUG("%s notification is sent to %s\n", Event_get_description(E), m->to);
 
       }
 
@@ -159,7 +157,7 @@ void handle_alert(Event_T E) {
         IS_EVENT_SET(m->events, Event_get_id(E)) &&
         (
           /* 'changed' event type is sent always */
-          E->id == EVENT_CHANGED ||
+          E->id == EVENT_CHANGED || E->id == EVENT_MATCH ||
           /* state change notification is sent always */
           E->state_changed       ||
           /* in the case that the state is failed for more cycles we check
@@ -174,21 +172,13 @@ void handle_alert(Event_T E) {
         Mail_T tmp= NULL;
 
         NEW(tmp);
-
         copy_mail(tmp, m);
-
-        if(Event_get_message(E))
-          tmp->opt_message= xstrdup(Event_get_message(E));
-
-        substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
-          Event_get_action_description(E));
-
+        substitute(&tmp, E);
         replace_bare_linefeed(&tmp);
-
         tmp->next= list;
         list= tmp;
 
-        DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
+        DEBUG("%s notification is sent to %s\n", Event_get_description(E), m->to);
 
       }
 
@@ -196,7 +186,9 @@ void handle_alert(Event_T E) {
 
     if(list) {
 
-      sendmail(list);
+      if(!sendmail(list)) {
+        rv = HANDLER_ALERT;
+      }
       gc_mail_list(&list);
 
     }
@@ -205,34 +197,36 @@ void handle_alert(Event_T E) {
 
   }
 
+  return rv;
+
 }
 
 
-static void substitute(Mail_T *m, char *name, const char *event,
-		       const char *action) {
+static void substitute(Mail_T *m, Event_T e) {
 
-  char host[STRLEN];
-  char now[STRLEN];
+  char timestamp[STRLEN];
 
-  ASSERT(m && name && event && action);
+  ASSERT(m && e);
 
-  Util_getRFC822Date(NULL, now, STRLEN);
-  if(gethostname(host, sizeof(host)) < 0) {
-    snprintf(host, STRLEN, "%s", LOCALHOST);
-  }
-  Util_replaceString(&(*m)->from, "$HOST", host);
-  Util_replaceString(&(*m)->subject, "$DATE", now);
-  Util_replaceString(&(*m)->message, "$DATE", now);
-  Util_replaceString(&(*m)->subject, "$HOST", host);
-  Util_replaceString(&(*m)->message, "$HOST", host);
-  Util_replaceString(&(*m)->subject, "$PROGRAM", name);
-  Util_replaceString(&(*m)->message, "$PROGRAM", name);
-  Util_replaceString(&(*m)->subject, "$SERVICE", name);
-  Util_replaceString(&(*m)->message, "$SERVICE", name);
-  Util_replaceString(&(*m)->subject, "$EVENT", event);
-  Util_replaceString(&(*m)->message, "$EVENT", event);
-  Util_replaceString(&(*m)->subject, "$ACTION", action);
-  Util_replaceString(&(*m)->message, "$ACTION", action);
+  Util_replaceString(&(*m)->from,    "$HOST", Run.localhostname);
+  Util_replaceString(&(*m)->subject, "$HOST", Run.localhostname);
+  Util_replaceString(&(*m)->message, "$HOST", Run.localhostname);
+  
+  Util_getRFC822Date(&e->collected, timestamp, STRLEN);
+  Util_replaceString(&(*m)->subject, "$DATE", timestamp);
+  Util_replaceString(&(*m)->message, "$DATE", timestamp);
+
+  Util_replaceString(&(*m)->subject, "$SERVICE", Event_get_source_name(e));
+  Util_replaceString(&(*m)->message, "$SERVICE", Event_get_source_name(e));
+
+  Util_replaceString(&(*m)->subject, "$EVENT", Event_get_description(e));
+  Util_replaceString(&(*m)->message, "$EVENT", Event_get_description(e));
+
+  Util_replaceString(&(*m)->subject, "$DESCRIPTION", NVLSTR(Event_get_message(e)));
+  Util_replaceString(&(*m)->message, "$DESCRIPTION", NVLSTR(Event_get_message(e)));
+
+  Util_replaceString(&(*m)->subject, "$ACTION", Event_get_action_description(e));
+  Util_replaceString(&(*m)->message, "$ACTION", Event_get_action_description(e));
 }
 
 
@@ -259,8 +253,6 @@ static void copy_mail(Mail_T n, Mail_T o) {
       Run.MailFormat.message?
       xstrdup(Run.MailFormat.message):
       xstrdup(ALERT_MESSAGE);
-  n->opt_message= NULL;
-  
 }
 
 
@@ -268,9 +260,4 @@ static void replace_bare_linefeed(Mail_T *m) {
   
   Util_replaceString(&(*m)->message, "\r\n", "\n");
   Util_replaceString(&(*m)->message, "\n", "\r\n");
-  if((*m)->opt_message) {
-    Util_replaceString(&(*m)->opt_message, "\r\n", "\n");
-    Util_replaceString(&(*m)->opt_message, "\n", "\r\n");
-  }
-  
 }
