@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -60,7 +60,7 @@
  *  @author Rory Toma, <rory@digeo.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: control.c,v 1.97 2005/09/05 09:51:37 martinp Exp $
+ *  @version \$Id: control.c,v 1.100 2006/01/01 22:44:01 martinp Exp $
  *
  *  @file
  */
@@ -87,38 +87,40 @@ static void  do_depend(Service_T, int);
  * @param action A string describing the action to execute
  */
 void control_service_daemon(const char *S, const char *action) {
-
+  
   Socket_T s;
   char *auth= Util_getBasicAuthHeader();
-
+  
   ASSERT(S);
   ASSERT(action);
-
+  
   if(Util_getAction(action) == ACTION_IGNORE) {
     log("%s: Cannot %s service '%s' -- invalid action %s\n",
         prog, action, S, action);
     goto error;
   }
-
+  
   s= socket_new(Run.bind_addr?Run.bind_addr:"localhost",
-		Run.httpdport, SOCKET_TCP, Run.httpdssl, NET_TIMEOUT);
+                Run.httpdport, SOCKET_TCP, Run.httpdssl, NET_TIMEOUT);
   
   if(!s) {
     
-    log("%s: Cannot connect to the monit daemon. "
-          "Did you start it with http support?\n", prog);
+    log("%s: Cannot connect to the monit daemon."
+        " Did you start it with http support?\n", prog);
     goto error;
     
   } else {
-
-    socket_print(s,
-      "GET /%s?action=%s HTTP/1.0\r\n%s\r\n", S, action, auth?auth:"");
+    
+    if(socket_print(s, "GET /%s?action=%s HTTP/1.0\r\n%s\r\n", 
+                    S, action, auth?auth:"") < 0)
+      log("%s: Cannot send the command '%s' to the monit daemon -- %s", 
+          prog, action?action:"null", STRERROR);
     socket_free(&s);
-      
+    
   }
   
-  error:
-  FREE(auth);
+error:
+    FREE(auth);
   
 }
 
