@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -23,7 +23,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_HPUX.c,v 1.11 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: sysdep_HPUX.c,v 1.13 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -46,6 +46,14 @@
 #include <mntent.h>
 #endif
 
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include "monitor.h"
 #include "device_sysdep.h"
 
