@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -75,7 +75,7 @@ static void set_environment(void);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: env.c,v 1.29 2005/01/06 20:51:49 martinp Exp $
+ *  @version \$Id: env.c,v 1.31 2006/01/01 22:44:01 martinp Exp $
  * 
  *  @file
  */
@@ -180,7 +180,6 @@ static void set_environment(void) {
 
   char cwd[MAXPATHLEN];
   struct passwd *pw;
-  char localhost[STRLEN];
   
   /* Get password struct */
   if ( ! (pw= getpwuid(geteuid())) ) {
@@ -198,13 +197,6 @@ static void set_environment(void) {
     
   }
   
-  /*
-   * Get the localhost name
-   */
-  if(gethostname(localhost, sizeof(localhost)) < 0) {
-    snprintf(localhost, STRLEN, "%s", LOCALHOST);
-  }
-
   /*
    * Save and clear the file creation mask
    */
@@ -216,7 +208,6 @@ static void set_environment(void) {
   Run.Env.cwd=  xstrdup(cwd);
   Run.Env.home= xstrdup(pw->pw_dir);
   Run.Env.user= xstrdup(pw->pw_name);
-  Run.localhostname= xstrdup(localhost);
   
 }
 
