@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -68,7 +68,7 @@ static void _gc_request(Request_T *);
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: gc.c,v 1.61 2005/08/05 09:38:09 chopp Exp $
+ *  @version \$Id: gc.c,v 1.66 2006/01/01 22:44:01 martinp Exp $
  *
  *  @file
  */
@@ -103,7 +103,12 @@ void gc() {
   if(Run.collectors)
     _gc_collector(&Run.collectors);
 
+  if(Run.eventlist)
+    _gc_event(&Run.eventlist);
+  
+  FREE(Run.eventlist_dir);
   FREE(Run.mygroup);
+  FREE(Run.localhostname);
   FREE(Run.httpsslpem);
   FREE(Run.httpsslclientpem);
   FREE(Run.bind_addr);
@@ -125,7 +130,6 @@ void gc_mail_list(Mail_T *m) {
   FREE((*m)->from);
   FREE((*m)->subject);
   FREE((*m)->message);
-  FREE((*m)->opt_message);
   FREE(*m);
 
 }
@@ -219,12 +223,21 @@ static void _gc_service(Service_T *s) {
   if((*s)->action_PPID)
     _gc_eventaction(&(*s)->action_PPID);
   
+  if((*s)->action_MONIT_START)
+    _gc_eventaction(&(*s)->action_MONIT_START);
+  
+  if((*s)->action_MONIT_STOP)
+    _gc_eventaction(&(*s)->action_MONIT_STOP);
+  
+  if((*s)->action_MONIT_RELOAD)
+    _gc_eventaction(&(*s)->action_MONIT_RELOAD);
+  
   if((*s)->eventlist)
     _gc_event(&(*s)->eventlist);
   
   FREE((*s)->name);
-  FREE((*s)->path);
   FREE((*s)->group);
+  FREE((*s)->path);
   
   (*s)->next= NULL;
 
@@ -322,7 +335,10 @@ static void _gc_event(Event_T *e) {
     _gc_event(&(*e)->next);
 
   pthread_mutex_destroy(&(*e)->mutex);
+
   (*e)->action= NULL;
+  FREE((*e)->source);
+  FREE((*e)->group);
   FREE((*e)->message);
   FREE(*e);
 
