@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -77,6 +77,7 @@
 #define VALIDATE    "/_validate"
 #define TEST        "/_monit"
 #define ABOUT       "/_about"
+#define PING        "/_ping"
 #define PIXEL       "/_pixel"
 #define STATUS      "/_status"
 #define RUN         "/_runtime"
@@ -92,9 +93,11 @@ static void do_home_system(HttpRequest, HttpResponse);
 static void do_home_device(HttpRequest, HttpResponse);
 static void do_home_directory(HttpRequest, HttpResponse);
 static void do_home_file(HttpRequest, HttpResponse);
+static void do_home_fifo(HttpRequest, HttpResponse);
 static void do_home_process(HttpRequest, HttpResponse);
 static void do_home_host(HttpRequest, HttpResponse);
 static void do_about(HttpRequest, HttpResponse);
+static void do_ping(HttpRequest, HttpResponse);
 static void not_found(HttpRequest, HttpResponse);
 static void do_runtime(HttpRequest, HttpResponse);
 static void do_viewlog(HttpRequest, HttpResponse);
@@ -114,6 +117,7 @@ static void print_service_rules_size(HttpResponse, Service_T);
 static void print_service_rules_match(HttpResponse, Service_T);
 static void print_service_rules_checksum(HttpResponse, Service_T);
 static void print_service_rules_process(HttpResponse, Service_T);
+static void print_service_rules_resource(HttpResponse, Service_T);
 static void print_service_params_port(HttpResponse, Service_T);
 static void print_service_params_icmp(HttpResponse, Service_T);
 static void print_service_params_perm(HttpResponse, Service_T);
@@ -125,8 +129,8 @@ static void print_service_params_size(HttpResponse, Service_T);
 static void print_service_params_match(HttpResponse, Service_T);
 static void print_service_params_checksum(HttpResponse, Service_T);
 static void print_service_params_process(HttpResponse, Service_T);
+static void print_service_params_resource(HttpResponse, Service_T);
 static void print_status(HttpRequest, HttpResponse);
-static void status_system_txt(HttpResponse, short);
 static void status_service_txt(Service_T, HttpResponse, short);
 static char *get_service_status_html(Service_T);
 static char *get_service_status_text(Service_T);
@@ -142,7 +146,7 @@ static char *get_service_status_text(Service_T);
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: cervlet.c,v 1.191 2005/09/25 21:02:54 martinp Exp $
+ *  @version \$Id: cervlet.c,v 1.202 2006/01/08 21:36:42 martinp Exp $
  *
  *  @file
  */
@@ -198,6 +202,8 @@ static void doGet(HttpRequest req, HttpResponse res) {
     do_viewlog(req, res);
   } else if(ACTION(ABOUT)) {
     do_about(req, res);
+  } else if(ACTION(PING)) {
+    do_ping(req, res);
   } else if(ACTION(STOP)) {
     if(is_readonly(req)) {
       send_error(res, SC_FORBIDDEN,
@@ -268,12 +274,11 @@ static void do_home(HttpRequest req, HttpResponse res) {
 
   FREE(uptime);
   
-  if(Run.doprocess) {
-    do_home_system(req, res);
-  }
+  do_home_system(req, res);
   do_home_process(req, res);
   do_home_device(req, res);
   do_home_file(req, res);
+  do_home_fifo(req, res);
   do_home_directory(req, res);
   do_home_host(req, res);
   
@@ -290,7 +295,7 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "monit " VERSION "</font></a></center><br><br>");
   out_print(res,
     "<ul>"
-    "<li>Copyright &copy; 2000-2005 by <a "
+    "<li>Copyright &copy; 2000-2006 by <a "
     "href=\"http://www.tildeslash.com/monit/who.php\">the monit "
     "project group</a>. All Rights Reserved.<br> </small><p>"
     "<li>Portions of this software are copyright &copy; 1995, 1996 "
@@ -310,18 +315,23 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "<a href='http://www.gnu.org/licenses/gpl.html'>GNU "
     "General Public License</a> for more details."
-    "<p><br><center><p><i>Please consider making a "
+    "<p></br><center><i><b>Please consider making a "
     "<a class=grey href=\"http://www.tildeslash.com/monit/donate.php\">"
-    "donation</a> and support our continued work with monit.</i></center>");
+    "donation</a> and support our continued work with monit.</b></i></center></p>");
   out_print(res,
-   "<p align=left><form method=GET action='/'><input type=submit"
-   " value='<<&nbsp; Back'></form></p>");
+    "<p></br><center>[Click <a href=\"/\">here</a> to go back to monit]"
+    "</center></p>");
 
   FOOT
 
 }
 
 
+static void do_ping(HttpRequest req, HttpResponse res) {
+  out_print(res, "pong");
+}
+
+
 static void do_runtime(HttpRequest req, HttpResponse res) {
 
   int pid=  exist_daemon();
@@ -358,6 +368,18 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
 	    "<tr><td>Use syslog</td><td>%s</td></tr>",
 	    Run.use_syslog?"True":"False");
 
+  if(Run.eventlist_dir) {
+    char slots[STRLEN];
+    if(Run.eventlist_slots < 0)
+      snprintf(slots, STRLEN, "unlimited");
+    else
+      snprintf(slots, STRLEN, "%d", Run.eventlist_slots);
+    out_print(res,
+              "<tr><td>Event queue</td>"
+              "<td>base directory %s with %d slots</td></tr>",
+              Run.eventlist_dir, Run.eventlist_slots);
+  }
+
   if(Run.collectors) {
     Collector_T c= Run.collectors;
     out_print(res, "<tr><td>Collector server(s)</td><td>");
@@ -519,7 +541,7 @@ static void do_viewlog(HttpRequest req, HttpResponse res) {
 	int n;
 	char buf[BUFSIZE+1];
 	
-	out_print(res, "<br><p><form><textarea cols=80 rows=30>");
+	out_print(res, "<br><p><form><textarea cols=120 rows=30 readonly>");
 	
 	while((n= fread(buf, sizeof(char), BUFSIZE, f)) > 0) {
 	  
@@ -696,15 +718,18 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
       "<tr>"
       "<td>Name</td>"
       "<td>%s</td>"
-      "</tr>"
-      "<tr>"
-      "<td>%s</td>"
-      "<td>%s</td>"
       "</tr>",
       servicetypes[s->type],
-      s->name,
-      pathnames[s->type],
-      s->path);
+      s->name);
+
+    if(s->path && *s->path)
+      out_print(res,
+        "<tr>"
+        "<td>%s</td>"
+        "<td>%s</td>"
+        "</tr>",
+        pathnames[s->type],
+        s->path);
 
     status= get_service_status_html(s);
     out_print(res,
@@ -779,6 +804,7 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
     print_service_params_match(res, s);
     print_service_params_checksum(res, s);
     print_service_params_process(res, s);
+    print_service_params_resource(res, s);
 
     /* Rules */
     print_service_rules_icmp(res, s);
@@ -792,6 +818,7 @@ static void do_service(HttpRequest req, HttpResponse res, Service_T s) {
     print_service_rules_match(res, s);
     print_service_rules_checksum(res, s);
     print_service_rules_process(res, s);
+    print_service_rules_resource(res, s);
 
     print_alerts(res, s->maillist);
 
@@ -838,37 +865,55 @@ static void not_found(HttpRequest req, HttpResponse res) {
 
 static void do_home_system(HttpRequest req, HttpResponse res) {
   
+  char      *status;
+  Service_T  s = Run.system;
+
+  status= get_service_status_html(s);
+
   out_print(res,
     "<br><p>&nbsp;</p>"
-    "<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+    "<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
     "<tr>"
     "<td width=\"20%%\"><h3><b>System</b></h3></td>"
-    "<td align=\"right\"><h3><b>Load</b></h3></td>"
-    "<td align=\"right\"><h3><b>CPU</b></h3></td>"
-    "<td align=\"right\"><h3><b>Memory</b></h3></td>"
+    "<td align=\"left\"><h3><b>Status</b></h3></td>");
+
+  if(Run.doprocess) {
+    out_print(res,
+      "<td align=\"right\"><h3><b>Load</b></h3></td>"
+      "<td align=\"right\"><h3><b>CPU</b></h3></td>"
+      "<td align=\"right\"><h3><b>Memory</b></h3></td>");
+  }
+
+  out_print(res,
     "</tr>"
     "<tr bgcolor=\"#EFEFEF\">"
-    "<td align=\"left\">%s</td>"
-    "<td align=\"right\" width=\"20%%\">[%.2f]&nbsp;[%.2f]&nbsp;[%.2f]</td>"
-    "<td align=\"right\" width=\"20%%\">"
+    "<td align=\"left\"><a href='/%s'>%s</a></td>"
+    "<td align=\"left\">%s</td>",
+    s->name, s->name,
+    status);
+  FREE(status);
+
+  if(Run.doprocess) {
+    out_print(res,
+      "<td align=\"right\" width=\"20%%\">[%.2f]&nbsp;[%.2f]&nbsp;[%.2f]</td>"
+      "<td align=\"right\" width=\"20%%\">"
       "%.1f%%us,&nbsp;%.1f%%sy"
-  #ifdef HAVE_CPU_WAIT
+    #ifdef HAVE_CPU_WAIT
       ",&nbsp;%.1f%%wa"
-  #endif
-    "</td>"
-    "<td align=\"right\" width=\"20%%\">%.1f%% [%ld&nbsp;kB]</td>"
+    #endif
+      "</td>"
+      "<td align=\"right\" width=\"20%%\">%.1f%% [%ld&nbsp;kB]</td>",
+      systeminfo.loadavg[0], systeminfo.loadavg[1], systeminfo.loadavg[2],
+      systeminfo.total_cpu_user_percent/10., systeminfo.total_cpu_syst_percent/10.,
+    #ifdef HAVE_CPU_WAIT
+      systeminfo.total_cpu_wait_percent/10.,
+    #endif
+      systeminfo.total_mem_percent/10., systeminfo.total_mem_kbyte);
+  }
+
+  out_print(res,
     "</tr>"
-    "</table>",
-    Run.localhostname,
-    systeminfo.loadavg[0], systeminfo.loadavg[1], systeminfo.loadavg[2],
-    systeminfo.total_cpu_user_percent/10.,
-    systeminfo.total_cpu_syst_percent/10.,
-  #ifdef HAVE_CPU_WAIT
-    systeminfo.total_cpu_wait_percent/10.,
-  #endif
-    systeminfo.total_mem_percent/10.,
-    systeminfo.total_mem_kbyte);
-    
+    "</table>");
 }
 
 
@@ -887,7 +932,7 @@ static void do_home_process(HttpRequest req, HttpResponse res) {
       
       out_print(res,
         "<br><p>&nbsp;</p>"
-        "<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+        "<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
         "<tr>"
         "<td width=\"20%%\"><h3><b>Process</b></h3></td>"
         "<td align=\"left\"><h3><b>Status</b></h3></td>"
@@ -974,7 +1019,7 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
       
       out_print(res,
 	"<br><p>&nbsp;</p>"
-	"<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+	"<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
 	"<tr>"
 	"<td width=\"20%%\"><h3><b>Device</b></h3></td>"
 	"<td align=\"left\"><h3><b>Status</b></h3></td>"
@@ -1053,7 +1098,7 @@ static void do_home_file(HttpRequest req, HttpResponse res) {
       
       out_print(res,
 	"<br><p>&nbsp;</p>"
-	"<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+	"<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
 	"<tr>"
 	"<td width=\"20%%\"><h3><b>File</b></h3></td>"
 	"<td align=\"left\"><h3><b>Status</b></h3></td>"
@@ -1111,6 +1156,75 @@ static void do_home_file(HttpRequest req, HttpResponse res) {
 }
 
 
+static void do_home_fifo(HttpRequest req, HttpResponse res) {
+  
+  Service_T  s;
+  char      *status;
+  int        on= TRUE;
+  int        header= TRUE;
+  
+  for(s= servicelist_conf; s; s= s->next_conf) {
+    
+    if(s->type != TYPE_FIFO) continue;
+    
+    if(header) {
+      
+      out_print(res,
+	"<br><p>&nbsp;</p>"
+	"<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
+	"<tr>"
+	"<td width=\"20%%\"><h3><b>Fifo</b></h3></td>"
+	"<td align=\"left\"><h3><b>Status</b></h3></td>"
+	"<td align=\"right\"><h3><b>Permission</b></h3></td>"
+	"<td align=\"right\"><h3><b>UID</b></h3></td>"
+	"<td align=\"right\"><h3><b>GID</b></h3></td>"
+	"</tr>");
+      
+      header= FALSE;
+      
+    }
+
+    status= get_service_status_html(s);
+    out_print(res,
+      "<tr %s>"
+      "<td width=\"20%%\"><a href='/%s'>%s</a></td>"
+      "<td align=\"left\">%s</td>",
+      on?"bgcolor=\"#EFEFEF\"":"",
+      s->name, s->name,
+      status);
+    FREE(status);
+    
+    if(!Util_hasServiceStatus(s)) {
+      
+      out_print(res,
+	"<td align=\"right\">-</td>"
+	"<td align=\"right\">-</td>"
+	"<td align=\"right\">-</td>");
+
+    } else {
+      
+      out_print(res,
+	"<td align=\"right\">%o</td>"
+	"<td align=\"right\">%d</td>"
+	"<td align=\"right\">%d</td>",
+	s->inf->st_mode & 07777,
+	s->inf->st_uid,
+	s->inf->st_gid);
+
+    }
+    
+    out_print(res, "</tr>");
+
+    on= on?FALSE:TRUE;
+    
+  }
+  
+  if(!header)
+    out_print(res, "</table>");
+  
+}
+
+
 static void do_home_directory(HttpRequest req, HttpResponse res) {
   
   Service_T        s;
@@ -1126,7 +1240,7 @@ static void do_home_directory(HttpRequest req, HttpResponse res) {
 
       out_print(res,
 	"<br><p>&nbsp;</p>"
-	"<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+	"<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
 	"<tr>"
 	"<td width=\"20%%\"><h3><b>Directory</b></h3></td>"
 	"<td align=\"left\"><h3><b>Status</b></h3></td>"
@@ -1197,7 +1311,7 @@ static void do_home_host(HttpRequest req, HttpResponse res) {
       
       out_print(res,
 	"<br><p>&nbsp;</p>"
-	"<table cellspacing=0 cellpadding=3 border=0 width=\"70%%\">"
+	"<table cellspacing=0 cellpadding=3 border=0 width=\"90%%\">"
 	"<tr>"
 	"<td width=\"20%%\"><h3><b>Host</b></h3></td>"
 	"<td align=\"left\"><h3><b>Status</b></h3></td>"
@@ -1299,6 +1413,8 @@ static void print_alerts(HttpResponse res, Mail_T s) {
 	  out_print(res, "Gid ");
       if(IS_EVENT_SET(r->events, EVENT_ICMP))
 	  out_print(res, "Icmp ");
+      if(IS_EVENT_SET(r->events, EVENT_INSTANCE))
+       out_print(res, "Instance ");
       if(IS_EVENT_SET(r->events, EVENT_INVALID))
 	  out_print(res, "Invalid ");
       if(IS_EVENT_SET(r->events, EVENT_MATCH))
@@ -1396,10 +1512,11 @@ static void print_service_rules_port(HttpResponse res, Service_T s) {
       if(p->family == AF_INET) {
         out_print(res,
           "<tr><td>Port</td><td>"
-          "If failed %s:%d%s [%s%s] with timeout %d seconds %s then %s "
+          "If failed %s:%d%s [%s via %s] with timeout %d seconds %s then %s "
           "else if passed %s then %s</td></tr>",
           p->hostname, p->port, p->request?p->request:"",
-          p->SSL.use_ssl?" via SSL":"", p->protocol->name, p->timeout,
+          p->protocol->name, Util_portTypeDescription(p),
+          p->timeout,
           ratio1, actionnames[a->failed->id],
           ratio2, actionnames[a->passed->id]);
         if(p->SSL.certmd5 != NULL)
@@ -1700,10 +1817,10 @@ static void print_service_rules_checksum(HttpResponse res, Service_T s) {
 
 static void print_service_rules_process(HttpResponse res, Service_T s) {
 
-  char ratio1[STRLEN];
-  char ratio2[STRLEN];
-
   if(s->type == TYPE_PROCESS) {
+
+    char ratio1[STRLEN];
+
     out_print(res, "<tr><td>Pid</td><td>If changed %s then %s</td></tr>\n",
       Util_getEventratio(s->action_PID->failed, ratio1),
       actionnames[s->action_PID->failed->id]);
@@ -1711,9 +1828,15 @@ static void print_service_rules_process(HttpResponse res, Service_T s) {
       Util_getEventratio(s->action_PPID->failed, ratio1),
       actionnames[s->action_PPID->failed->id]);
   }
+}
+
+
+static void print_service_rules_resource(HttpResponse res, Service_T s) {
 
   if(s->resourcelist) {
 
+    char          ratio1[STRLEN];
+    char          ratio2[STRLEN];
     Resource_T    q;
     EventAction_T a;
     
@@ -1737,6 +1860,39 @@ static void print_service_rules_process(HttpResponse res, Service_T s) {
           ratio2, actionnames[a->passed->id]);
 	break;
 	  
+      case RESOURCE_ID_CPUUSER: 
+	  
+	out_print(res,"<tr><td>CPU user limit</td>"
+	  "<td>If %s %.1f%% %s then %s "
+          "else if passed %s then %s</td></tr>", 
+	  operatornames[q->operator],
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
+	break;
+	  
+      case RESOURCE_ID_CPUSYSTEM: 
+	  
+	out_print(res,"<tr><td>CPU system limit</td>"
+	  "<td>If %s %.1f%% %s then %s "
+          "else if passed %s then %s</td></tr>", 
+	  operatornames[q->operator],
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
+	break;
+	  
+      case RESOURCE_ID_CPUWAIT: 
+	  
+	out_print(res,"<tr><td>CPU wait limit</td>"
+	  "<td>If %s %.1f%% %s then %s "
+          "else if passed %s then %s</td></tr>", 
+	  operatornames[q->operator],
+	  q->limit/10.0,
+          ratio1, actionnames[a->failed->id],
+          ratio2, actionnames[a->passed->id]);
+	break;
+	  
       case RESOURCE_ID_MEM_PERCENT: 
 	  
 	out_print(res,"<tr><td>Memory usage limit</td>"
@@ -1857,16 +2013,16 @@ static void print_service_params_port(HttpResponse res, Service_T s) {
           if(!p->is_available) {
             out_print(res,
               "<tr><td>Port Response time</font></td><td>"
-              "<font color='#ff0000'>connection failed to %s:%d%s [%s%s]</font>"
+              "<font color='#ff0000'>connection failed to %s:%d%s [%s via %s]</font>"
               "</td></tr>",
               p->hostname, p->port, p->request?p->request:"",
-              p->SSL.use_ssl?" via SSL":"", p->protocol->name);
+              p->protocol->name, Util_portTypeDescription(p));
           } else {
             out_print(res,
               "<tr><td>Port Response time</td>"
-              "<td>%.3fs to %s:%d%s [%s%s]</td></tr>",
+              "<td>%.3fs to %s:%d%s [%s via %s]</td></tr>",
               p->response, p->hostname, p->port, p->request?p->request:"",
-              p->SSL.use_ssl?" via SSL":"", p->protocol->name);
+              p->protocol->name, Util_portTypeDescription(p));
           }
         } else if(p->family == AF_UNIX) {
           if(!p->is_available) {
@@ -1920,6 +2076,7 @@ static void print_service_params_icmp(HttpResponse res, Service_T s) {
 static void print_service_params_perm(HttpResponse res, Service_T s) {
 
   if(s->type == TYPE_FILE ||
+     s->type == TYPE_FIFO ||
      s->type == TYPE_DIRECTORY ||
      s->type == TYPE_DEVICE) {
 
@@ -1942,6 +2099,7 @@ static void print_service_params_perm(HttpResponse res, Service_T s) {
 static void print_service_params_uid(HttpResponse res, Service_T s) {
 
   if(s->type == TYPE_FILE ||
+     s->type == TYPE_FIFO ||
      s->type == TYPE_DIRECTORY ||
      s->type == TYPE_DEVICE) {
 
@@ -1964,6 +2122,7 @@ static void print_service_params_uid(HttpResponse res, Service_T s) {
 static void print_service_params_gid(HttpResponse res, Service_T s) {
 
   if(s->type == TYPE_FILE ||
+     s->type == TYPE_FIFO ||
      s->type == TYPE_DIRECTORY ||
      s->type == TYPE_DEVICE) {
 
@@ -1985,7 +2144,9 @@ static void print_service_params_gid(HttpResponse res, Service_T s) {
 
 static void print_service_params_timestamp(HttpResponse res, Service_T s) {
 
-  if(s->type == TYPE_FILE || s->type == TYPE_DIRECTORY) {
+  if(s->type == TYPE_FILE ||
+     s->type == TYPE_FIFO ||
+     s->type == TYPE_DIRECTORY) {
 
     if(!Util_hasServiceStatus(s)) {
 
@@ -2133,15 +2294,6 @@ static void print_service_params_process(HttpResponse res, Service_T s) {
         "<tr><td>Parent process id </td><td>-</td></tr>"
         "<tr><td>Process uptime</td><td>-</td></tr>");
 
-      if(Run.doprocess) {
-        out_print(res,
-          "<tr><td>CPU usage</td><td>-</td></tr>"
-          "<tr><td>Memory usage</td><td>-</td></tr>"
-          "<tr><td>Children</td><td>-</td></tr>"
-          "<tr><td>Total CPU usage (incl. children)</td><td>-</td></tr>"
-          "<tr><td>Total memory usage (incl. children)</td><td>-</td></tr>");
-      }
-
     } else {
 
       char *uptime;
@@ -2158,8 +2310,32 @@ static void print_service_params_process(HttpResponse res, Service_T s) {
         "<tr><td>Process uptime</td><td>%s</td></tr>",
         uptime);
       FREE(uptime);
+    }
+  }
+}
 
-      if(Run.doprocess) {
+
+static void print_service_params_resource(HttpResponse res, Service_T s) {
+
+  if(Run.doprocess && (s->type == TYPE_PROCESS || s->type == TYPE_SYSTEM) ) {
+
+    if(!Util_hasServiceStatus(s)) {
+      if(s->type == TYPE_PROCESS) {
+        out_print(res,
+          "<tr><td>CPU usage</td><td>-</td></tr>"
+          "<tr><td>Memory usage</td><td>-</td></tr>"
+          "<tr><td>Children</td><td>-</td></tr>"
+          "<tr><td>Total CPU usage (incl. children)</td><td>-</td></tr>"
+          "<tr><td>Total memory usage (incl. children)</td><td>-</td></tr>");
+      } else if(s->type == TYPE_SYSTEM) {
+        out_print(res,
+          "<tr><td>Load average</td><td>-</td></tr>"
+          "<tr><td>CPU usage</td><td>-</td></tr>"
+          "<tr><td>Memory usage</td><td>-</td></tr>");
+      }
+    } else {
+
+      if(s->type == TYPE_PROCESS) {
         out_print(res,
           "<tr><td>CPU usage</td><td><font%s>%.1f%%</font></td></tr>",
           (s->error & EVENT_RESOURCE)?" color='#ff0000'":"",
@@ -2182,6 +2358,31 @@ static void print_service_params_process(HttpResponse res, Service_T s) {
           "<td><font%s>%.1f%% [%ldkB]</font></td></tr>",
           (s->error & EVENT_RESOURCE)?" color='#ff0000'":"",
           s->inf->total_mem_percent/10.0, s->inf->total_mem_kbyte);
+      } else if(s->type == TYPE_SYSTEM) {
+        out_print(res,
+          "<tr><td>Load average</td><td><font%s>[%.2f] [%.2f] [%.2f]</font></td></tr>",
+          (s->error & EVENT_RESOURCE)?" color='#ff0000'":"",
+          systeminfo.loadavg[0],
+          systeminfo.loadavg[1],
+          systeminfo.loadavg[2]);
+        out_print(res,
+          "<tr><td>CPU usage</td><td><font%s>%.1f%%us %.1f%%sy"
+        #ifdef HAVE_CPU_WAIT
+          " %.1f%%wa"
+        #endif
+          "%s",
+          (s->error & EVENT_RESOURCE)?" color='#ff0000'":"",
+          systeminfo.total_cpu_user_percent/10.,
+          systeminfo.total_cpu_syst_percent/10.,
+        #ifdef HAVE_CPU_WAIT
+          systeminfo.total_cpu_wait_percent/10.,
+        #endif
+          "</font></td></tr>");
+        out_print(res,
+          "<tr><td>Memory usage</td><td><font%s>%ld kB [%.1f%%]</font></td></tr>",
+          (s->error & EVENT_RESOURCE)?" color='#ff0000'":"",
+          systeminfo.total_mem_kbyte,
+          systeminfo.total_mem_percent/10.);
       }
     }
   }
@@ -2229,10 +2430,6 @@ static void print_status(HttpRequest req, HttpResponse res)
     out_print(res, "The monit daemon %s uptime: %s\n\n", VERSION, uptime);
     FREE(uptime);
 
-    if(Run.doprocess)
-    {
-      status_system_txt(res, level);
-    }
     for(s= servicelist_conf; s; s= s->next_conf)
     {
       status_service_txt(s, res, level);
@@ -2242,54 +2439,6 @@ static void print_status(HttpRequest req, HttpResponse res)
 }
 
 
-static void status_system_txt(HttpResponse res, short level) {
-  char time[STRLEN];
-
-  if(level == LEVEL_SUMMARY)
-  {
-    char prefix[STRLEN];
-
-    snprintf(prefix, STRLEN, "System '%s'", Run.localhostname);
-    out_print(res, "%-35s [%.2f] [%.2f] [%.2f]\n",
-      prefix,
-      systeminfo.loadavg[0],
-      systeminfo.loadavg[1],
-      systeminfo.loadavg[2]);
-  }
-  else
-  {
-    ctime_r(&systeminfo.collected, time);
-    out_print(res,
-      "System '%s'\n"
-      "  %-33s [%.2f] [%.2f] [%.2f]\n"
-      "  %-33s %.1f%%us %.1f%%sy"
-    #ifdef HAVE_CPU_WAIT
-      " %.1f%%wa"
-    #endif
-      "\n"
-      "  %-33s %ld kB [%.1f%%]\n"
-      "  %-33s %s\n",
-      Run.localhostname,
-      "load average",
-      systeminfo.loadavg[0],
-      systeminfo.loadavg[1],
-      systeminfo.loadavg[2],
-      "cpu",
-      systeminfo.total_cpu_user_percent/10.,
-      systeminfo.total_cpu_syst_percent/10.,
-    #ifdef HAVE_CPU_WAIT
-      systeminfo.total_cpu_wait_percent/10.,
-    #endif
-      "memory usage",
-      systeminfo.total_mem_kbyte,
-      systeminfo.total_mem_percent/10.,
-      "data collected",
-      time);
-  }
-
-}
-
-
 static void status_service_txt(Service_T s, HttpResponse res, short level) {
   char *status = get_service_status_text(s);
 
@@ -2313,7 +2462,9 @@ static void status_service_txt(Service_T s, HttpResponse res, short level) {
 	    "monitoring status", monitornames[s->monitor]);
 
     if(Util_hasServiceStatus(s)) {
-      if(s->type == TYPE_FILE || s->type == TYPE_DIRECTORY || 
+      if(s->type == TYPE_FILE ||
+         s->type == TYPE_FIFO || 
+         s->type == TYPE_DIRECTORY || 
          s->type == TYPE_DEVICE) {
         out_print(res,
   		"  %-33s %o\n"
@@ -2323,7 +2474,9 @@ static void status_service_txt(Service_T s, HttpResponse res, short level) {
   		"uid", (int)s->inf->st_uid,
   		"gid", (int)s->inf->st_gid);
       }
-      if(s->type == TYPE_FILE || s->type == TYPE_DIRECTORY) {
+      if(s->type == TYPE_FILE ||
+         s->type == TYPE_FIFO || 
+         s->type == TYPE_DIRECTORY) {
         ctime_r(&s->inf->timestamp, time);
         out_print(res,
   		"  %-33s %s",
@@ -2418,11 +2571,11 @@ static void status_service_txt(Service_T s, HttpResponse res, short level) {
         for(p= s->portlist; p; p= p->next) {
   	if(p->family == AF_INET) {
   	  out_print(res,
-  		    "  %-33s %.3fs to %s:%d%s [%s%s]\n",
+  		    "  %-33s %.3fs to %s:%d%s [%s via %s]\n",
   		    "port response time", p->is_available?p->response:-1., 
   		    p->hostname,
 		    p->port, p->request?p->request:"", p->protocol->name,
-  		    p->SSL.use_ssl?" via SSL":"");
+                    Util_portTypeDescription(p));
   	} else if(p->family == AF_UNIX) {
   	  out_print(res,
   		    "  %-33s %.3fs to %s [%s]\n",
@@ -2431,6 +2584,29 @@ static void status_service_txt(Service_T s, HttpResponse res, short level) {
   	}
         }
       }
+      if(s->type == TYPE_SYSTEM && Run.doprocess) {
+        out_print(res,
+          "  %-33s [%.2f] [%.2f] [%.2f]\n"
+          "  %-33s %.1f%%us %.1f%%sy"
+        #ifdef HAVE_CPU_WAIT
+          " %.1f%%wa"
+        #endif
+          "\n"
+          "  %-33s %ld kB [%.1f%%]\n",
+          "load average",
+          systeminfo.loadavg[0],
+          systeminfo.loadavg[1],
+          systeminfo.loadavg[2],
+          "cpu",
+          systeminfo.total_cpu_user_percent/10.,
+          systeminfo.total_cpu_syst_percent/10.,
+        #ifdef HAVE_CPU_WAIT
+          systeminfo.total_cpu_wait_percent/10.,
+        #endif
+          "memory usage",
+          systeminfo.total_mem_kbyte,
+          systeminfo.total_mem_percent/10.);
+      }
     }
     ctime_r(&s->collected, time);
     out_print(res, "  %-33s %s\n", "data collected", time);
