@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -80,7 +80,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: log.c,v 1.23 2005/01/06 20:51:49 martinp Exp $
+ *  @version \$Id: log.c,v 1.25 2006/01/01 22:44:01 martinp Exp $
  *
  *  @file
  */
@@ -90,7 +90,6 @@
 
 
 static FILE *LOG= NULL;
-static char time_str[STRLEN];
 static pthread_mutex_t log_mutex= PTHREAD_MUTEX_INITIALIZER;
 
 
@@ -98,7 +97,7 @@ static pthread_mutex_t log_mutex= PTHREAD_MUTEX_INITIALIZER;
 
 
 static int  open_log();
-static char *timestr(void);
+static char *timefmt(char *t, int size);
 
 
 /* ------------------------------------------------------------------ Public */
@@ -136,6 +135,7 @@ void log_log(const char *s, ...) {
   long len;
   va_list ap;
   char *msg= NULL;
+  char datetime[STRLEN];
 
   ASSERT(s);
   
@@ -152,7 +152,7 @@ void log_log(const char *s, ...) {
     
   } else if(LOG) {
     LOCK(log_mutex)
-      fprintf(LOG,"[%s] %s", timestr(), msg);
+      fprintf(LOG,"[%s] %s", timefmt(datetime, STRLEN), msg);
     END_LOCK;
     
   }
@@ -215,19 +215,19 @@ static int open_log() {
 
 /**
  * Returns the current time as a formated string, see the TIMEFORMAT
- * macro in the header file
+ * macro in monitor.h
  */
-static char *timestr() {
+static char *timefmt(char *t, int size) {
 
   time_t now;
   struct tm tm;
   
   time(&now);
-  if(!strftime(time_str, STRLEN, TIMEFORMAT, localtime_r(&now, &tm))) {
-    *time_str= 0;
+  if(!strftime(t, size, TIMEFORMAT, localtime_r(&now, &tm))) {
+    *t= 0;
   }
   
-  return time_str;
+  return t;
   
 }
 
