@@ -42,6 +42,10 @@
 #include <sys/param.h>
 #endif
 
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
+
 #include "md5.h"
 #include "monitor.h"
 
@@ -435,8 +439,14 @@ md5_crypt (key, md5_salt_prefix, salt, buffer, buflen)
      char *buffer;
      int buflen;
 {
+#ifdef __SUNPRO_C
+  /* Suns Forte Developer C Compiler does not support __aligned__ etc. */
+  unsigned char alt_result[16];
+#else
   unsigned char alt_result[16]
     __attribute__ ((__aligned__ (__alignof__ (md5_uint32))));
+#endif
+  
   struct md5_ctx ctx;
   struct md5_ctx alt_ctx;
   size_t salt_len;
@@ -455,6 +465,11 @@ md5_crypt (key, md5_salt_prefix, salt, buffer, buflen)
   salt_len = MIN (strcspn (salt, "$"), 8);
   key_len = strlen (key);
 
+#ifndef __SUNPRO_C
+  /* Suns Forte Developer C Compiler doesn`t support this below,
+     so lets hope 32 bit integers are well aligned.
+  */
+
   if ((key - (char *) 0) % __alignof__ (md5_uint32) != 0)
     {
       char *tmp = (char *) alloca (key_len + __alignof__ (md5_uint32));
@@ -474,6 +489,7 @@ md5_crypt (key, md5_salt_prefix, salt, buffer, buflen)
 		salt, salt_len);
       ASSERT ((salt - (char *) 0) % __alignof__ (md5_uint32) == 0);
     }
+#endif
 
   /* Prepare for the real work.  */
   md5_init_ctx (&ctx);
