@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -67,6 +67,7 @@
 #include "md5.h"
 #include "sha.h"
 #include "state.h"
+#include "event.h"
 
 
 /**
@@ -80,7 +81,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: monitor.c,v 1.121 2005/09/11 14:57:02 martinp Exp $
+ *  @version \$Id: monitor.c,v 1.128 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -331,6 +332,10 @@ static void do_reinit() {
   if(can_http())
     monit_http(START_HTTP);
 
+  /* send the monit startup notification */
+  Event_post(Run.system, EVENT_INSTANCE, STATE_FAILED,
+    Run.system->action_MONIT_RELOAD, "Monit reloaded");
+
 }
 
 
@@ -416,6 +421,10 @@ static void do_exit() {
 
     log("%s daemon with pid [%d] killed\n", prog, (int)getpid());
 
+    /* send the monit stop notification */
+    Event_post(Run.system, EVENT_INSTANCE, STATE_FAILED,
+      Run.system->action_MONIT_STOP, "Monit stopped");
+
   }
 
   /* wait for all wait_start threads to finish */
@@ -465,17 +474,31 @@ static void do_default() {
     if(can_http())
       monit_http(START_HTTP);
     
-    for(;;) {
+    /* send the monit startup notification */
+    Event_post(Run.system, EVENT_INSTANCE, STATE_FAILED,
+      Run.system->action_MONIT_START, "Monit started");
+
+    while(TRUE) {
 
       validate();
       State_save();
       sleep(Run.polltime);
 
-      if(Run.doreload)
-	do_reinit();
+      if(Run.dowakeup) {
+        Run.dowakeup = FALSE;
+        log("Awakened by User defined signal 1\n");
+      }
       
-      if(Run.stopped)
-	do_exit();
+      if(Run.stopped) {
+        do_exit();
+      } else if(Run.doreload) {
+        do_reinit();
+      } else {
+        Event_post(Run.system, EVENT_INSTANCE, STATE_PASSED,
+          Run.system->action_MONIT_START, "Monit has not changed");
+        Event_post(Run.system, EVENT_INSTANCE, STATE_PASSED,
+          Run.system->action_MONIT_RELOAD, "Monit has not changed");
+      }
       
     }
     
@@ -534,10 +557,6 @@ static void handle_options(int argc, char **argv) {
         Run.statefile= xstrdup(optarg);
         break;
 
-    case 'i':
-	log("%s: option -i is obsolete\n", prog);
-	break;
-      
     case 'I':
 	Run.init= TRUE;
 	break;
@@ -643,7 +662,7 @@ static void help() {
 static void version() {
   
   printf("This is monit version %s\n", VERSION);
-  printf("Copyright (C) 2000-2005 by the monit project group.");
+  printf("Copyright (C) 2000-2006 by the monit project group.");
   printf(" All Rights Reserved.\n");
  
 }
@@ -674,6 +693,6 @@ static RETSIGTYPE do_destroy(int sig) {
  */
 static RETSIGTYPE do_wakeup(int sig) {
 
-  log("Awakened by User defined signal 1\n");
+  Run.dowakeup= TRUE;
 
 }
