@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -77,6 +77,7 @@
 #define MYPIDDIR           PIDDIR
 #define MYPIDFILE          "monit.pid"
 #define MYSTATEFILE        "monit.state"
+#define MYEVENTLISTBASE    "/var/monit"
 
 #define LOCALHOST          "localhost"
 
@@ -128,6 +129,8 @@
 #define TYPE_FILE          2
 #define TYPE_PROCESS       3
 #define TYPE_HOST          4
+#define TYPE_SYSTEM        5
+#define TYPE_FIFO          6
 
 #define RESOURCE_ID_CPU_PERCENT       1
 #define RESOURCE_ID_MEM_PERCENT       2
@@ -140,6 +143,9 @@
 #define RESOURCE_ID_TOTAL_MEM_PERCENT 9
 #define RESOURCE_ID_INODE             10
 #define RESOURCE_ID_SPACE             11
+#define RESOURCE_ID_CPUUSER           12
+#define RESOURCE_ID_CPUSYSTEM         13
+#define RESOURCE_ID_CPUWAIT           14
 
 #define DIGEST_CLEARTEXT   1
 #define DIGEST_CRYPT       2
@@ -165,6 +171,11 @@
 #define LEVEL_NAME_FULL    "full"
 #define LEVEL_NAME_SUMMARY "summary"
 
+#define HANDLER_PASSED     0x0
+#define HANDLER_ALERT      0x1
+#define HANDLER_COLLECTOR  0x2
+#define HANDLER_MAX        HANDLER_COLLECTOR
+
 #define ATTEMPT_COUNT      3
 
 /** ------------------------------------------------- Special purpose macros */
@@ -192,6 +203,8 @@ Sigfunc *signal(int signo, Sigfunc * func);
 #define MIN(x,y) ((x) < (y) ? (x) : (y))
 #define IS(a,b)  ((a&&b)?!strcasecmp(a, b):0)
 #define DEBUG if(Run.debug) log
+#define FLAG(x, y) (x & y) == y 
+#define NVLSTR(x) (x?x:"")
 
 
 /** ------------------------------------------------- Synchronization macros */
@@ -295,7 +308,6 @@ typedef struct mymail {
   char *from;                                     /**< The mail from address */
   char *subject;                                       /**< The mail subject */
   char *message;                                       /**< The mail message */
-  char *opt_message;                 /**< An optional message used in alerts */
   unsigned int events;  /*< Events for which this mail object should be sent */
   unsigned int reminder;              /*< Send error reminder each Xth cycle */
 
@@ -360,63 +372,6 @@ typedef struct mysysteminfo {
 } SystemInfo_T;
 
 
-/** Defines data for application runtime */
-struct myrun {
-  int  stopped;         /**< TRUE if monit was stopped. Flag used by threads */
-  char *controlfile;                /**< The file to read configuration from */
-  char *logfile;                         /**< The file to write logdata into */
-  char *localhostname;                      /**< The host name for localhost */
-  char *pidfile;                                  /**< This programs pidfile */
-  char *statefile;                /**< The file with the saved runtime state */
-  char *mygroup;                              /**< Group Name of the Service */
-  int  debug;                   /**< Write debug information - TRUE or FALSE */
-  int  use_syslog;                          /**< If TRUE write log to syslog */
-  int  dolog;       /**< TRUE if program should log actions, otherwise FALSE */
-  int  isdaemon;                 /**< TRUE if program should run as a daemon */
-  int  polltime;        /**< In deamon mode, the sleeptime (sec) between run */
-  int  dohttpd;                    /**< TRUE if monit HTTP server should run */
-  int  httpdssl;                     /**< TRUE if monit HTTP server uses ssl */
-  char *httpsslpem;                       /**< PEM file for the HTTPS server */
-  int  clientssl;   /**< TRUE if monit HTTP server uses ssl with client auth */
-  char *httpsslclientpem;      /**< PEM file/dir to check against at connect */
-  int  allowselfcert;   /**< TRUE if self certified client certs are allowed */
-  int  httpdsig;   /**< TRUE if monit HTTP server presents version signature */
-  int  httpdport;                    /**< The monit http server's portnumber */
-  int  once;                                       /**< TRUE - run only once */
-  int  init;                   /**< TRUE - don't background to run from init */
-  int  facility;              /** The facility to use when running openlog() */
-  int  doprocess;                 /**< TRUE if process status engine is used */
-  char *bind_addr;                  /**< The address monit http will bind to */
-  int  doreload;             /**< TRUE if a monit daemon should reinitialize */
-  mode_t umask;                /**< The initial umask monit was started with */
-  int  testing;   /**< Running in configuration testing mode - TRUE or FALSE */
-  int  wait_start;         /**< Number of threads waiting for process' start */
-  time_t incarnation;              /**< Unique ID for running monit instance */
-
-       /** An object holding program relevant "environment" data, see; env.c */
-  struct myenvironment {
-    char *user;             /**< The the effective user running this program */
-    char *home;                                    /**< Users home directory */
-    char *cwd;                                /**< Current working directory */
-  } Env;
-
-  int mailserver_timeout;    /**< Connect and read timeout for a SMTP server */
-  Mail_T maillist;                /**< Global alert notification mailinglist */
-  MailServer_T mailservers;    /**< List of MTAs used for alert notification */
-  Collector_T collectors;  /**< Event notification and status receivers list */
-  Auth_T credentials;    /** A list holding Basic Authentication information */
-                                      /** User selected standard mail format */
-  struct myformat {
-    char *from;                          /**< The standard mail from address */
-    char *subject;                            /**< The standard mail subject */
-    char *message;                            /**< The standard mail message */
-  } MailFormat;
-                                          
-  pthread_mutex_t mutex;    /**< Mutex used for service data synchronization */
-
-};
-
-
 /** Defines a protocol object with protocol functions */
 typedef struct myprotocol {
   const char *name;                                       /**< Protocol name */
@@ -642,10 +597,10 @@ typedef struct myinfo {
   int    children;
   long   mem_kbyte;    
   long   total_mem_kbyte;
-  int    mem_percent;                                    /**< pecentage * 10 */
-  int    total_mem_percent;                              /**< pecentage * 10 */
-  int    cpu_percent;                                    /**< pecentage * 10 */
-  int    total_cpu_percent;                              /**< pecentage * 10 */
+  int    mem_percent;                                   /**< percentage * 10 */
+  int    total_mem_percent;                             /**< percentage * 10 */
+  int    cpu_percent;                                   /**< percentage * 10 */
+  int    total_cpu_percent;                             /**< percentage * 10 */
   time_t uptime;                                         /**< Process uptime */
 } *Info_T;
 
@@ -699,26 +654,36 @@ typedef struct myservice {
   EventAction_T action_NONEXIST;   /**< Description of the action upon event */
   EventAction_T action_TIMEOUT;    /**< Description of the action upon event */
 
+  /** Internal monit events */
+  EventAction_T action_MONIT_START;         /**< Monit instance start action */
+  EventAction_T action_MONIT_STOP;           /**< Monit instance stop action */
+  EventAction_T action_MONIT_RELOAD;       /**< Monit instance reload action */
+
   /** Runtime parameters */
   unsigned long long error;                          /**< Error flags bitmap */
   Info_T             inf;                          /**< Service check result */
   time_t             collected;                /**< When were data collected */
 
-
   /** Events */
   struct myevent {
+    #define           EVENT_VERSION  1      /**< The event structure version */
     int               id;                      /**< The event identification */
     time_t            collected;                 /**< When the event occured */
-    struct myservice *source;                              /**< Event source */
+    char             *source;                 /**< Event source service name */
+    char             *group;                         /**< Service group name */
+    int               mode;             /**< Monitoring mode for the service */
+    int               type;                      /**< Monitored service type */
     short             state;            /**< TRUE if failed, FALSE if passed */
     short             state_changed;              /**< TRUE if state changed */
     long long         state_map;           /**< Event bitmap for last cycles */
     unsigned int      count;                             /**< The event rate */
+    unsigned int      flag;                     /**< The handlers state flag */
     char             *message;    /**< Optional message describing the event */
     EventAction_T     action;           /**< Description of the event action */
     /** For internal use */
     pthread_mutex_t   mutex;      /**< Mutex used for action synchronization */
     struct myevent   *next;                         /**< next event in chain */
+    struct myevent   *previous;                 /**< previous event in chain */
   } *eventlist;                                     /**< Pending events list */
 
   /** Context specific parameters */
@@ -735,6 +700,71 @@ typedef struct myservice {
 typedef struct myevent *Event_T;
 
 
+/** Defines data for application runtime */
+struct myrun {
+  int  stopped;         /**< TRUE if monit was stopped. Flag used by threads */
+  char *controlfile;                /**< The file to read configuration from */
+  char *logfile;                         /**< The file to write logdata into */
+  char *localhostname;                      /**< The host name for localhost */
+  char *pidfile;                                  /**< This programs pidfile */
+  char *statefile;                /**< The file with the saved runtime state */
+  char *mygroup;                              /**< Group Name of the Service */
+  int  debug;                   /**< Write debug information - TRUE or FALSE */
+  int  use_syslog;                          /**< If TRUE write log to syslog */
+  int  dolog;       /**< TRUE if program should log actions, otherwise FALSE */
+  int  isdaemon;                 /**< TRUE if program should run as a daemon */
+  int  polltime;        /**< In deamon mode, the sleeptime (sec) between run */
+  int  dohttpd;                    /**< TRUE if monit HTTP server should run */
+  int  httpdssl;                     /**< TRUE if monit HTTP server uses ssl */
+  char *httpsslpem;                       /**< PEM file for the HTTPS server */
+  int  clientssl;   /**< TRUE if monit HTTP server uses ssl with client auth */
+  char *httpsslclientpem;      /**< PEM file/dir to check against at connect */
+  int  allowselfcert;   /**< TRUE if self certified client certs are allowed */
+  int  httpdsig;   /**< TRUE if monit HTTP server presents version signature */
+  int  httpdport;                    /**< The monit http server's portnumber */
+  int  once;                                       /**< TRUE - run only once */
+  int  init;                   /**< TRUE - don't background to run from init */
+  int  facility;              /** The facility to use when running openlog() */
+  int  doprocess;                 /**< TRUE if process status engine is used */
+  char *bind_addr;                  /**< The address monit http will bind to */
+  int  doreload;             /**< TRUE if a monit daemon should reinitialize */
+  int  dowakeup;           /**< TRUE if a monit daemon was wake up by signal */
+  mode_t umask;                /**< The initial umask monit was started with */
+  int  testing;   /**< Running in configuration testing mode - TRUE or FALSE */
+  int  wait_start;         /**< Number of threads waiting for process' start */
+  time_t incarnation;              /**< Unique ID for running monit instance */
+  int  handler_init;                  /**< The handlers queue initialization */
+  int  handler_flag;                            /**< The handlers state flag */
+  int  handler_queue[HANDLER_MAX+1];         /**< The handlers queue counter */
+  Service_T system;                          /**< The general system service */
+  char *eventlist_dir;                   /**< The event queue base directory */
+  int  eventlist_slots;          /**< The event queue size - number of slots */
+
+       /** An object holding program relevant "environment" data, see; env.c */
+  struct myenvironment {
+    char *user;             /**< The the effective user running this program */
+    char *home;                                    /**< Users home directory */
+    char *cwd;                                /**< Current working directory */
+  } Env;
+
+  int mailserver_timeout;    /**< Connect and read timeout for a SMTP server */
+  Mail_T maillist;                /**< Global alert notification mailinglist */
+  MailServer_T mailservers;    /**< List of MTAs used for alert notification */
+  Collector_T collectors;  /**< Event notification and status receivers list */
+  Auth_T credentials;    /** A list holding Basic Authentication information */
+  Event_T eventlist;              /** A list holding partialy handled events */
+                                      /** User selected standard mail format */
+  struct myformat {
+    char *from;                          /**< The standard mail from address */
+    char *subject;                            /**< The standard mail subject */
+    char *message;                            /**< The standard mail message */
+  } MailFormat;
+                                          
+  pthread_mutex_t mutex;    /**< Mutex used for service data synchronization */
+
+};
+
+
 /* -------------------------------------------------------- Global variables */
 
 char   *prog;                                  /**< The Name of this Program */
@@ -761,7 +791,7 @@ void  control_service_daemon(const char *, const char *);
 void  setup_dependants();
 void  reset_depend();
 void  spawn(Service_T, Command_T, const char *);
-void  status(char *);
+int   status(char *);
 int   log_init() ;
 void  log_log(const char *, ...) ;
 void  log_close();
@@ -771,7 +801,7 @@ void  gc();
 void  gc_mail_list(Mail_T *);
 int   kill_daemon(int);
 int   exist_daemon(); 
-void  sendmail(Mail_T);
+int   sendmail(Mail_T);
 int   sock_msg(int, char *, ...);
 void  init_env();
 void *xmalloc (int);
@@ -795,13 +825,15 @@ int  check_device(Service_T);
 int  check_file(Service_T);
 int  check_directory(Service_T);
 int  check_remote_host(Service_T);
+int  check_system(Service_T);
+int  check_fifo(Service_T);
 int  check_URL(Service_T s);
 int  sha_md5_stream (FILE *, void *, void *);
 void reset_procinfo(Service_T);
 int  check_service_status(Service_T);
 void printhash(char *);  
 char *status_xml(Event_T, short);
-void handle_collector(Event_T);
+int  handle_collector(Event_T);
 int  do_wakeupcall();
 
 #endif
