@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -134,7 +134,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.62 2005/10/05 21:37:47 martinp Exp $
+ *  @version \$Id: net.c,v 1.71 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -156,17 +156,28 @@ static unsigned short checksum_ip(unsigned char *, int);
  * @return TRUE if hostname resolves, otherwise FALSE
  */
 int check_host(const char *hostname) {
-  
+
+#ifdef HAVE_GETADDRINFO        
+  struct addrinfo *res;
+#else
   struct hostent *hp;
+#endif  
 
   ASSERT(hostname);
 
-  if((hp = gethostbyname(hostname)) == NULL) {
+#ifdef HAVE_GETADDRINFO        
+  if(getaddrinfo(hostname, NULL, NULL, &res)!=0) {
     return FALSE;
   }
-  
+  freeaddrinfo(res);
+#else
+  if((hp = gethostbyname(hostname)) == NULL) {
+          return FALSE;
+  }
+#endif  
+
   return TRUE;
-  
+
 }
 
 
@@ -183,38 +194,33 @@ int check_socket(int socket) {
 
 
 /**
- * Verify that the udp socket is ready for i|o. The given socket must
- * be a connected udp socket if we should be able to test the udp
- * server. The test is conducted by sending one byte to the server and
- * check for a returned ICMP error when reading from the socket. A
- * better test would be to send an empty SYN udp package to avoid
- * possibly raising an error from the server we are testing but
- * assembling an udp by hand requires SOCKET_RAW and running the
- * program as root.
+ * Verify that the udp server is up. The given socket must be a 
+ * connected udp socket if we should be able to test the udp server. 
+ * The test is conducted by sending a datagram to the server and
+ * check for a returned ICMP error when reading from the socket.
  * @param socket A socket
  * @return TRUE if the socket is ready, otherwise FALSE.
  */
 int check_udp_socket(int socket) {
+  
+  char buf[STRLEN]= {0};
 
-  int r;
-  char buf[1]= {0};
-
-  /*
-   * R/W is asynchronous and we should probably loop and wait longer
-   * for a possible ICMP error.
+  /* We have to send something and if the UDP server is down/unreachable
+   *  the remote host should send an ICMP error. We then need to call read
+   *  to get the ICMP error as a ECONNREFUSED errno. This test is asynchronous
+   *  so we must wait, but we do not want to block to long either and it is
+   *  probably better to report a server falsely up than to block to long.
    */
-  write(socket, buf, 1);
-  sleep(2);
-  r= read(socket, buf, 1);
-  if(0>r) {
+  sock_write(socket, buf, 1, 0);
+  if(sock_read(socket, buf, STRLEN, 2) < 0) {
     switch(errno) {
-    case ECONNREFUSED: return FALSE;
-    default:           break;
+      case ECONNREFUSED: return FALSE;
+      default:           break;
     }
   }
   
   return TRUE;
-
+  
 }
 
 
@@ -513,9 +519,10 @@ int can_write(int socket, int timeout) {
  * @param socket the socket to write to
  * @param buffer The buffer to write
  * @param size Number of bytes to send
+ * @param timeout Seconds to wait for data to be written
  * @return The number of bytes sent or -1 if an error occured.
  */
-int sock_write(int socket, const void *buffer, int size) {
+int sock_write(int socket, const void *buffer, int size, int timeout) {
 
   ssize_t n= 0;
   
@@ -528,7 +535,7 @@ int sock_write(int socket, const void *buffer, int size) {
   } while(n == -1 && errno == EINTR);
   
   if(n == -1 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
-    if(! can_write(socket, NET_TIMEOUT)) {
+    if(! can_write(socket, timeout)) {
       return -1;
     }
     do {
@@ -550,7 +557,7 @@ int sock_write(int socket, const void *buffer, int size) {
  * @param size Number of bytes to read from the socket
  * @param timeout Seconds to wait for data to be available
  * @return The number of bytes read or -1 if an error occured. 
-*/
+ */
 int sock_read(int socket, void *buffer, int size, int timeout) {
   
   ssize_t n;
@@ -577,6 +584,46 @@ int sock_read(int socket, void *buffer, int size, int timeout) {
 }
 
 
+/**
+ * Write <code>size</code> bytes from the <code>buffer</code> to the
+ * <code>socket</code>. The given socket <b>must</b> be a connected
+ * UDP socket
+ * @param socket the socket to write to
+ * @param buffer The buffer to write
+ * @param size Number of bytes to send
+ * @param timeout Seconds to wait for data to be written
+ * @return The number of bytes sent or -1 if an error occured.
+ */
+int udp_write(int socket, void *b, int len, int timeout) {
+  
+  int i, n;
+  
+  ASSERT(timeout>=0);
+  
+  for(i= 4; i>=1; i--) {
+    
+    do {
+      n= sock_write(socket, b, len, 0);
+    } while(n == -1 && errno == EINTR);
+    
+    if(n == -1 && (errno != EAGAIN || errno != EWOULDBLOCK))
+      return -1;
+    
+    /* Simple retransmit scheme, wait for the server to reply 
+    back to our socket. This assume a request-response pattern, 
+    which really is the only pattern we can support */
+    if(can_read(socket, timeout/i)) return n;
+    DEBUG("udp_write: Resending request\n");
+    
+  }
+  
+  errno= EHOSTUNREACH;
+  
+  return -1;
+  
+}
+
+
 /**
  * Create a ICMP socket against hostname, send echo and wait for response.
  * The 'count' echo requests  is send and we expect at least one reply.
