@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -32,7 +32,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.h,v 1.26 2005/08/05 11:21:18 martinp Exp $
+ *  @version \$Id: net.h,v 1.29 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -62,14 +62,10 @@ int check_socket(int socket);
 
 
 /**
- * Verify that the udp socket is ready for i|o. The given socket must
- * be a connected udp socket if we should be able to test the udp
- * server. The test is conducted by sending one byte to the server and
- * check for a returned ICMP error when reading from the socket. A
- * better test would be to send an empty SYN udp package to avoid
- * possibly raising an error from the server we are testing but
- * assembling an udp by hand requires SOCKET_RAW and running the
- * program as root.
+ * Verify that the udp server is up. The given socket must be a 
+ * connected udp socket if we should be able to test the udp server. 
+ * The test is conducted by sending a datagram to the server and
+ * check for a returned ICMP error when reading from the socket.
  * @param socket A socket
  * @return TRUE if the socket is ready, otherwise FALSE.
  */
@@ -171,10 +167,11 @@ int can_write(int socket, int timeout);
  * @param socket the socket to write to
  * @param buffer The buffer to write
  * @param size Number of bytes to send
+ * @param timeout Seconds to wait for data to be written
  * @return The number of bytes sent or -1 if an error occured.
  */
-int sock_write(int socket, const void *buffer, int size);
-
+int sock_write(int socket, const void *buffer, int size, int timeout);
+  
 
 /**
  * Read up to size bytes from the <code>socket</code> into the
@@ -188,6 +185,20 @@ int sock_write(int socket, const void *buffer, int size);
 */
 int sock_read(int socket, void *buffer, int size, int timeout);
 
+
+/**
+ * Write <code>size</code> bytes from the <code>buffer</code> to the
+ * <code>socket</code>. The given socket <b>must</b> be a connected
+ * UDP socket
+ * @param socket the socket to write to
+ * @param buffer The buffer to write
+ * @param size Number of bytes to send
+ * @param timeout Seconds to wait for data to be written
+ * @return The number of bytes sent or -1 if an error occured.
+ */
+int udp_write(int socket, void *b, int len, int timeout);
+
+
 /**
  * Create a ICMP socket against hostname, send echo and wait for response.
  * The 'count' echo requests  is send and we expect at least one reply.
