@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -95,7 +95,7 @@ static int page_size;
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Joe Bryant <JBryant@RiteAid.com>
- *  @version \$Id: sysdep_HPUX.c,v 1.15 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: sysdep_HPUX.c,v 1.18 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -182,17 +182,16 @@ int getloadavg_sysdep (double *a, int na) {
  */
 int initprocesstree_sysdep(ProcessTree_T ** reference) {
 
-  int treesize;
-  int i;
-
-  ProcessTree_T *  pt;
+  int            i;
+  int            treesize;
+  ProcessTree_T *pt;
 
   ASSERT(reference);
 
   /* Gather process data */
 
   if ((treesize=pstat_getproc(psall,
-                              (size_t) NPROCS* sizeof(pst_status),
+                              (size_t) NPROCS* sizeof(struct pst_status),
                               (size_t) NPROCS, 0))==-1) {
 
     return 0;
@@ -205,26 +204,28 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
 
   /* Inspect data */
 
-  for (i = 0; i < tree_size; pp++, i++) {
+  for (i = 0; i < treesize; i++) {
 
-    pt[i].pid = psall[i]->pst_pid;
-    pt[i].ppid = psall[i]->pst_ppid;
+    pt[i].pid = psall[i].pst_pid;
+    pt[i].ppid = psall[i].pst_ppid;
 
     /* get the actual time */
 
     pt[i].time = get_float_time();
 
-    /* jiffies -> seconds = 1 / HZ
-       HZ is defined in "asm/param.h"  and it is usually 1/100s but on
-       alpha system it is 1/1024s */
+    /*
+     * jiffies -> seconds = 1 / HZ
+     * HZ is defined in "asm/param.h"  and it is usually 1/100s but on
+     * alpha system it is 1/1024s
+     */
 
-    pt[i].cputime =  ( psall[i]->pst_utime + psall[i]->pst_stime ) * 10 / HZ;
-    pt[i].mem_kbyte = (unsigned long)(((double)psall[i]->pst_rssize)/1024.0 *
+    pt[i].cputime =  ( psall[i].pst_utime + psall[i].pst_stime ) * 10 / HZ;
+    pt[i].mem_kbyte = (unsigned long)(((double)psall[i].pst_rssize)/1024.0 *
                                page_size);
 
     /* State is Zombie -> then we are a Zombie ... clear or? (-: */
 
-    if ( psall[i]->pst_stat || PS_ZOMBIE ) {
+    if ( psall[i].pst_stat || PS_ZOMBIE ) {
 
       pt[i].status_flag |= PROCESS_ZOMBIE;
       
