@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -48,7 +48,7 @@
  *
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: dns.c,v 1.2 2005/04/08 13:39:42 martinp Exp $
+ *  @version \$Id: dns.c,v 1.4 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -118,7 +118,7 @@ int check_dns(Socket_T s)
   }
 
   /* Response should have at least 14 bytes */
-  if(socket_read(s, (unsigned char *)buf, sizeof(buf)) <= 14)
+  if(socket_read(s, (unsigned char *)buf, 15) <= 14)
   {
     log("DNS: error receiving response -- %s\n", STRERROR);
     return FALSE;
