@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -40,7 +40,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Michael Amster, <mamster@webeasy.com>
  *
- *  @version \$Id: ftp.c,v 1.20 2005/09/10 12:47:51 hauk Exp $
+ *  @version \$Id: ftp.c,v 1.24 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -51,7 +51,7 @@ int check_ftp(Socket_T s) {
 
   ASSERT(s);
 
-  if(! socket_readln(s, buf, STRLEN)) {
+  if(!socket_readln(s, buf, STRLEN)) {
     log("FTP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
@@ -64,17 +64,12 @@ int check_ftp(Socket_T s) {
     return FALSE;
   }
 
-  /* Read past banners */
-  while(NULL != socket_readln(s, buf, STRLEN)) {
-    if(!Util_startsWith(buf, "220")) break;
-  }
-  
   if(socket_print(s, "QUIT\r\n") < 0) {
     log("FTP: error sending data -- %s\n", STRERROR);
     return FALSE;
   }
 
-  if(socket_read(s, buf, STRLEN) <= 0) {
+  if(!socket_readln(s, buf, STRLEN)) {
     log("FTP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
