@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -43,7 +43,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: imap.c,v 1.17 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: imap.c,v 1.19 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -56,7 +56,7 @@ int check_imap(Socket_T s) {
   
   ASSERT(s);
   
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(s, buf, sizeof(buf))) {
     log("IMAP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
@@ -73,7 +73,7 @@ int check_imap(Socket_T s) {
     return FALSE;
   }
 
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(s, buf, sizeof(buf))) {
     log("IMAP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
