@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -49,7 +49,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: ldap3.c,v 1.13 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: ldap3.c,v 1.15 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -127,7 +127,7 @@ int check_ldap3(Socket_T s) {
     return FALSE;
   }
 
-  if(socket_read(s, (unsigned char *)buf, sizeof(buf)) <= 0) {
+  if(socket_read(s, (unsigned char *)buf, sizeof(response)) <= 0) {
     log("LDAP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
