@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -44,13 +44,13 @@
  *
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: mysql.c,v 1.3 2005/04/04 14:58:30 martinp Exp $
+ *  @version \$Id: mysql.c,v 1.7 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
 int check_mysql(Socket_T s) {
 
-  unsigned char  buf[STRLEN];
+  unsigned char buf[STRLEN];
 
   unsigned char requestLogin[10] = {
     0x06,                                 /** Packet Length */
@@ -104,17 +104,18 @@ int check_mysql(Socket_T s) {
 
   ASSERT(s);
 
-  if(socket_read(s, (unsigned char *)buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(s, (char *)buf, sizeof(buf))) {
     log("MYSQL: error receiving greeting -- %s\n", STRERROR);
     return FALSE;
   }
 
-  if(socket_write(s, (unsigned char *)requestLogin, sizeof(requestLogin)) < 0) {
+  if(socket_write(s, requestLogin, sizeof(requestLogin)) < 0) {
     log("MYSQL: error sending login -- %s\n", STRERROR);
     return FALSE;
   }
 
-  if(socket_read(s, (unsigned char *)buf, sizeof(buf)) <= 6) {
+  /* read just first few bytes  which contains enough information */
+  if(socket_read(s, buf, 7) <= 6) {
     log("MYSQL: error receiving login response -- %s\n", STRERROR);
     return FALSE;
   }
@@ -136,7 +137,7 @@ int check_mysql(Socket_T s) {
       return FALSE;
     }
 
-    if(socket_read(s, (unsigned char *)buf, sizeof(buf)) <= 0) {
+    if(socket_read(s, buf, sizeof(responsePing)) <= 0) {
       log("MYSQL: error receiving ping response -- %s\n", STRERROR);
       return FALSE;
     }
