@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -46,6 +46,7 @@
 #define P_MYSQL          17
 #define P_DNS            18
 #define P_POSTFIXPOLICY  19
+#define P_TNS            20
 
 void  gc_protocols();
 
@@ -69,6 +70,7 @@ void* create_smtp();
 void* create_ssh();
 void* create_rdate();
 void* create_rsync();
+void* create_tns();
 
 /* "Package" locale Protocol routines */
 int check_apache_status(Socket_T);
@@ -90,6 +92,7 @@ int check_smtp(Socket_T);
 int check_ssh(Socket_T);
 int check_rdate(Socket_T);
 int check_rsync(Socket_T);
+int check_tns(Socket_T);
 
 
 #endif
