@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -40,7 +40,7 @@
  * 
  *  @author Igor Homyakov <homyakov@altlinux.ru>
  *
- *  @version \$Id: rsync.c,v 1.10 2005/01/06 20:51:50 martinp Exp $
+ *  @version \$Id: rsync.c,v 1.13 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -53,7 +53,7 @@ int check_rsync(Socket_T s) {
   
   ASSERT(s);
     
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(s, buf, sizeof(buf))) {
     log("RSYNC: did not see server greeting  -- %s\n", STRERROR);
     return FALSE;
   }
@@ -86,7 +86,7 @@ int check_rsync(Socket_T s) {
     return FALSE;
   }
 
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(s, buf, sizeof(buf))) {
     log("RSYNC: did not see server answer  -- %s\n", STRERROR);
     return FALSE;
   }
