@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -34,59 +34,56 @@
 #include "protocol.h"
 
 /**
- *  Check the server for greeting code 220 and then send QUIT and
+*  Check the server for greeting code 220 and then send QUIT and
  *  check for code 221. If alive return TRUE, else, return FALSE.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Michael Amster, <mamster@webeasy.com>
  *
- *  @version \$Id: smtp.c,v 1.22 2005/09/26 19:23:06 martinp Exp $
+ *  @version \$Id: smtp.c,v 1.26 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
 int check_smtp(Socket_T s) {
-
+  
   int status;
-  char separator;
   char buf[STRLEN];
-
+  
   ASSERT(s);
-
-  /* Read (multiline) SMTP greeting */
-  do {
-    if(socket_readln(s, buf, sizeof(buf)) <= 0) {
-      log("SMTP: error receiving data -- %s\n", STRERROR);
-      return FALSE;
-    }
-    Util_chomp(buf);
-    sscanf(buf, "%d%c%*s", &status, &separator);
-    if(status != 220) {
-      log("SMTP error: %s\n", buf);
-      return FALSE;
-    }
-  } while(separator == '-');
-
+  
+  if(!socket_readln(s, buf, sizeof(buf))) {
+    log("SMTP: error receiving data -- %s\n", STRERROR);
+    return FALSE;
+  }
+  
+  Util_chomp(buf);
+  
+  sscanf(buf, "%d%*s", &status);
+  if(status != 220) {
+    log("SMTP error: %s\n", buf);
+    return FALSE;
+  }
   
   /* Terminate the session */
   if(socket_print(s, "QUIT\r\n") < 0) {
     log("SMTP: error sending data -- %s\n", STRERROR);
     return FALSE;
   }
-
+  
   /* Check the reply */
-  do {
-    if(socket_readln(s, buf, sizeof(buf)) <= 0) {
-      log("SMTP: error receiving data -- %s\n", STRERROR);
-      return FALSE;
-    }
-    Util_chomp(buf);
-    sscanf(buf, "%d%c%*s", &status, &separator);
-    if(status != 221) {
-      log("SMTP error: %s\n", buf);
-      return FALSE;
-    }
-  } while(separator == '-');
-
+  if(!socket_readln(s, buf, sizeof(buf))) {
+    log("SMTP: error receiving data -- %s\n", STRERROR);
+    return FALSE;
+  }
+  
+  Util_chomp(buf);
+  
+  sscanf(buf, "%d%*s", &status);
+  if(status != 221) {
+    log("SMTP error: %s\n", buf);
+    return FALSE;
+  }
+  
   return TRUE;
   
 }
