@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -66,7 +66,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: sendmail.c,v 1.35 2005/01/06 20:51:49 martinp Exp $
+ *  @version \$Id: sendmail.c,v 1.42 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -98,16 +98,18 @@ static void do_send(SendMail_T *S, const char *, ...);
 /**
  * Send mail messages via SMTP
  * @param mail A Mail object
+ * @return FALSE if failed, TRUE if passed
  */
-void sendmail(Mail_T mail) {
+int sendmail(Mail_T mail) {
 
   Mail_T m;
   SendMail_T S;
   char now[STRLEN];
+  int rv;
   
   ASSERT(mail);
   
-  if(sigsetjmp(S.error, TRUE)) {
+  if((rv = sigsetjmp(S.error, TRUE))) {
     goto exit;
   }
   open_server(&S);
@@ -115,10 +117,6 @@ void sendmail(Mail_T mail) {
   if(gethostname(S.localhost, sizeof(S.localhost)) < 0) {
     snprintf(S.localhost, sizeof(S.localhost), "%s", LOCALHOST);
   }
-  /*
-   * FIXME: Implement this as a small state machine and handle server
-   * returned status codes as transition points.
-   */
   do_status(&S);
   do_send(&S, "HELO %s\r\n", S.localhost); 
   do_status(&S);
@@ -139,9 +137,6 @@ void sendmail(Mail_T mail) {
     do_send(&S, "Content-Transfer-Encoding: quoted-printable\r\n");
     do_send(&S, "\r\n");
     do_send(&S, "%s\r\n", m->message);
-    if(m->opt_message) {
-      do_send(&S, "%s\r\n", m->opt_message);
-    }
     do_send(&S, ".\r\n");
     do_status(&S);
   }
@@ -150,6 +145,7 @@ exit:
   if(S.socket) {
     socket_free(&S.socket);
   }
+  return !rv;
 }
 
 
@@ -180,7 +176,7 @@ static void do_status(SendMail_T *S) {
   int  status;
   char buf[STRLEN];
   
-  if (socket_read(S->socket, buf, sizeof(buf)) <= 0) {
+  if(!socket_readln(S->socket, buf, sizeof(buf))) {
     log("Sendmail: error receiving data from the mailserver '%s' -- %s\n",
 	S->server, STRERROR);
     siglongjmp(S->error, TRUE);
@@ -213,8 +209,7 @@ static void open_server(SendMail_T *S) {
       log("Trying the next mail server '%s:%i'\n", S->server, S->port);
       continue;
     } else {
-      log("No mail servers are available, aborting alert\n");
-      /* FIXME: Add mail to a queue for later retry */
+      log("No mail servers are available\n");
       siglongjmp(S->error, TRUE);
     }
   } while(TRUE);
