@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -50,15 +50,16 @@
  * Implementation of the socket interface.
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- * @version \$Id: socket.c,v 1.44 2005/06/08 22:54:17 hauk Exp $
+ * @version \$Id: socket.c,v 1.56 2006/01/01 22:44:02 martinp Exp $
  * @file
  */
 
 
 /* ------------------------------------------------------------- Definitions */
 
-#define TYPE_LOCAL  0
-#define TYPE_ACCEPT 1
+#define TYPE_LOCAL   0
+#define TYPE_ACCEPT  1
+#define RBUFFER_SIZE 1024
 
 struct Socket_T {
   int port;
@@ -70,9 +71,18 @@ struct Socket_T {
   int connection_type;
   ssl_connection *ssl;
   ssl_server_connection *sslserver;
+  int length;
+  int offset;
+  unsigned char buffer[RBUFFER_SIZE+1];
 };
 
 
+/* -------------------------------------------------------------- Prototypes */
+
+
+static int fill(Socket_T S, int timeout);
+
+
 /* ------------------------------------------------------------------ Public */
 
 
@@ -92,9 +102,9 @@ struct Socket_T {
  */
 Socket_T socket_new(const char *host, int port, int type, int use_ssl,
                     int timeout) {
-
+  
   Ssl_T ssl= {use_ssl, SSL_VERSION_AUTO, NULL};
-
+  
   return socket_create_t(host, port, type, ssl, timeout);
   
 }
@@ -106,18 +116,20 @@ Socket_T socket_new(const char *host, int port, int type, int use_ssl,
  * @return The connected Socket or NULL if an error occurred
  */
 Socket_T socket_create(void *port) {
-
+  
   int s;
   Port_T p= port;
-
+  
   ASSERT(port);
-
+  
   if((s= create_generic_socket(p)) != -1) {
-
+    
     Socket_T S= NULL;
-
+    
     NEW(S);
     S->socket= s;
+    S->length= 0;
+    S->offset= 0;
     S->type= p->type;
     S->port= p->port;
     S->timeout= p->timeout;
@@ -128,33 +140,33 @@ Socket_T socket_create(void *port) {
     } else {
       S->host= xstrdup(p->hostname);
     }
-
+    
     if(p->SSL.use_ssl) {
       if(! (S->ssl= new_ssl_connection(NULL, p->SSL.version))) {
-	goto ssl_error;
+        goto ssl_error;
       }
       if(! embed_ssl_socket(S->ssl, S->socket)) {
-	goto ssl_error;
+        goto ssl_error;
       }
       if(p->SSL.certmd5) {
-	if(! check_ssl_md5sum(S->ssl, p->SSL.certmd5)) {
-	  goto ssl_error;
-	}
+        if(! check_ssl_md5sum(S->ssl, p->SSL.certmd5)) {
+          goto ssl_error;
+        }
       }
     }
-
+    
     S->Port= port;
-
+    
     return S;
-
-    ssl_error:
-    socket_free(&S);
+    
+ssl_error:
+      socket_free(&S);
     return NULL;
-
+    
   }
-
+  
   return NULL;
-
+  
 }
 
 
@@ -170,8 +182,8 @@ Socket_T socket_create(void *port) {
  * @return The connected Socket or NULL if an error occurred
  */
 Socket_T socket_create_t(const char *host, int port, int type, Ssl_T ssl,
-			 int timeout) {
-
+                         int timeout) {
+  
   int s;
   int proto= type==SOCKET_UDP?SOCK_DGRAM:SOCK_STREAM;
   
@@ -181,13 +193,15 @@ Socket_T socket_create_t(const char *host, int port, int type, Ssl_T ssl,
     ASSERT(type==SOCKET_TCP);
   }
   ASSERT(timeout>0);
-
+  
   if((s= create_socket(host, port, proto, timeout)) != -1) {
     
     Socket_T S= NULL;
- 
+    
     NEW(S);
     S->socket= s;
+    S->length= 0;
+    S->offset= 0;
     S->port= port;
     S->type= proto;
     S->timeout= timeout;
@@ -196,28 +210,28 @@ Socket_T socket_create_t(const char *host, int port, int type, Ssl_T ssl,
     
     if(ssl.use_ssl) {
       if(! (S->ssl= new_ssl_connection(NULL, ssl.version))) {
-	goto ssl_error;
+        goto ssl_error;
       }
       if(! embed_ssl_socket(S->ssl, S->socket)) {
-	goto ssl_error;
+        goto ssl_error;
       }
       if(ssl.certmd5) {
-	if(! check_ssl_md5sum(S->ssl, ssl.certmd5)) {
-	  goto ssl_error;
-	}
+        if(! check_ssl_md5sum(S->ssl, ssl.certmd5)) {
+          goto ssl_error;
+        }
       }
     }
     
     return S;
     
-    ssl_error:
-    socket_free(&S);
+ssl_error:
+      socket_free(&S);
     return NULL;
-
+    
   }
   
   return NULL;
-
+  
 }
 
 
@@ -235,14 +249,16 @@ Socket_T socket_create_t(const char *host, int port, int type, Ssl_T ssl,
  * @return A Socket or NULL if an error occurred
  */
 Socket_T socket_create_a(int socket, const char *remote_host,
-			 int port, void *sslserver) {
-
+                         int port, void *sslserver) {
+  
   Socket_T S;
-
+  
   ASSERT(socket>=0);
   ASSERT(remote_host);
-
+  
   NEW(S);
+  S->length= 0;
+  S->offset= 0;
   S->port= port;
   S->socket= socket;
   S->type= SOCK_STREAM;
@@ -262,8 +278,8 @@ Socket_T socket_create_a(int socket, const char *remote_host,
   
   return S;
   
-  ssl_error:
-  socket_free(&S);
+ssl_error:
+    socket_free(&S);
   return NULL;
   
 }
@@ -275,9 +291,9 @@ Socket_T socket_create_a(int socket, const char *remote_host,
  * @param S A Socket object reference
  */
 void socket_free(Socket_T *S) {
-
+  
   ASSERT(S && *S);
-
+  
 #ifdef HAVE_OPENSSL
   if((*S)->ssl && (*S)->ssl->handler) {
     if((*S)->connection_type==TYPE_LOCAL) {
@@ -288,7 +304,7 @@ void socket_free(Socket_T *S) {
     }
   }
 #endif
-
+  
   close_socket((*S)->socket);
   FREE((*S)->host);
   FREE(*S);
@@ -296,7 +312,7 @@ void socket_free(Socket_T *S) {
 }
 
 
-/* -------------------------------------------------------------- Properties */
+/* ------------------------------------------------------------ Properties */
 
 
 /**
@@ -305,23 +321,23 @@ void socket_free(Socket_T *S) {
  * @return TRUE if the socket is ready otherwise FALSE 
  */
 int socket_is_ready(Socket_T S) {
-
+  
   ASSERT(S);
   
   switch(S->type) {
-
+    
     case SOCK_STREAM:
       return check_socket(S->socket);
-
+      
     case SOCK_DGRAM:
       return check_udp_socket(S->socket);
-
+      
     default:
       break;
   }
-
+  
   return FALSE;
-
+  
 }
 
 
@@ -331,11 +347,11 @@ int socket_is_ready(Socket_T S) {
  * @return The socket descriptor
  */
 int socket_get_socket(Socket_T S) {
-
+  
   ASSERT(S);
   
   return S->socket;
-
+  
 }
 
 
@@ -345,11 +361,11 @@ int socket_get_socket(Socket_T S) {
  * @return The socket type
  */
 int socket_get_type(Socket_T S) {
-
+  
   ASSERT(S);
   
   return S->type;
-
+  
 }
 
 
@@ -360,11 +376,11 @@ int socket_get_type(Socket_T S) {
  * @return The Port object or NULL
  */
 void *socket_get_Port(Socket_T S) {
-
+  
   ASSERT(S);
   
   return S->Port;
-
+  
 }
 
 
@@ -374,11 +390,11 @@ void *socket_get_Port(Socket_T S) {
  * @return The remote host's port number
  */
 int socket_get_remote_port(Socket_T S) {
-
+  
   ASSERT(S);
   
   return S->port;
-
+  
 }
 
 
@@ -393,11 +409,11 @@ const char *socket_get_remote_host(Socket_T S) {
   ASSERT(S);
   
   return S->host;
-
+  
 }
 
 
-/* ------------------------------------------------------------------ Public */
+/* ---------------------------------------------------------------- Public */
 
 
 /**
@@ -408,15 +424,15 @@ const char *socket_get_remote_host(Socket_T S) {
  * @return The bytes sent or -1 if an error occured
  */
 int socket_print(Socket_T S, const char *m, ...) {
-
+  
   int n;
   long l;
   va_list ap;
   char *buf= NULL;
-
+  
   ASSERT(S);
   ASSERT(m);
-
+  
   va_start(ap, m);
   buf= Util_formatString(m, ap, &l);
   va_end(ap);
@@ -425,7 +441,7 @@ int socket_print(Socket_T S, const char *m, ...) {
   FREE(buf);
   
   return n;
-
+  
 }
 
 
@@ -437,33 +453,59 @@ int socket_print(Socket_T S, const char *m, ...) {
  * @return The bytes sent or -1 if an error occured
  */
 int socket_write(Socket_T S, void *b, int size) {
-
+  
   int n= 0;
   void *p= b;
   
   ASSERT(S);
+  
+  /* Clear any extra data read from the server */
+  socket_reset(S);
 
   while(size > 0) {
     
     if(S->ssl) {
-      n= send_ssl_socket(S->ssl, p, size);
+      n= send_ssl_socket(S->ssl, p, size, S->timeout);
     } else {
-      n= sock_write(S->socket,  p, size);
+      if(S->type==SOCK_DGRAM)
+        n= udp_write(S->socket,  p, size, S->timeout);
+      else
+        n= sock_write(S->socket,  p, size, S->timeout);
     }
-
+    
     if(n <= 0) break;
     p+= n;
     size-= n;
-
+    
   }
-
+  
   if(n < 0) {
     /* No write or a partial write is an error */
     return -1;
   }
   
   return  (int)(p - b);
+  
+}
 
+
+/**
+ * Read a single byte. The byte is returned as an int in the range 0
+ * to 255.
+ * @param S A Socket_T object
+ * @return The byte read, or -1 if the end of the stream has been reached
+ */
+int socket_read_byte(Socket_T S) {
+  
+  ASSERT(S);
+  
+  if(S->offset >= S->length) {
+    if(fill(S, S->timeout) <= 0)
+      return -1;
+  }
+  
+  return S->buffer[S->offset++];
+  
 }
 
 
@@ -475,70 +517,118 @@ int socket_write(Socket_T S, void *b, int size) {
  * @return The bytes read or -1 if an error occured
  */
 int socket_read(Socket_T S, void *b, int size) {
-
-  int n= 0;
-  void *p= b;
-  int timeout= 0;
+  
+  int c;
+  unsigned char *p= b;
   
   ASSERT(S);
   
-  timeout= S->timeout;
-  
-  while(size > 0) {
-    if(S->ssl) {
-      n= recv_ssl_socket(S->ssl, p, size, timeout);
-    } else {
-      n= sock_read(S->socket, p, size, timeout);
-    }
-    if(n <= 0) break;
-    p+= n;
-    size-= n;
-    /* We managed to read some bytes and we only keep on reading
-     * available data, otherwise we return with what we have read
-     */
-    timeout= 0;
-  }
-
-  if(n < 0 && p==b) {
-    return -1;
+  while((size-- > 0) && ((c= socket_read_byte(S)) >= 0)) { 
+    *p++= c;
   }
   
-  return (int)(p - b);
- 
+  return  (int)p - (int)b;
+  
 }
- 
+
 
 /**
  * Reads in at most one less than size <code>characters</code> and
  * stores them into the buffer pointed to by s. Reading stops after
  * an EOF or a newline.  If a newline is read, it is stored into the
- * buffer.  A '\0' is stored after the last character in the buffer.
- * @unnecessary S A Socket_T object
- * @unnecessary s A character buffer to store the string in
+ * buffer.   A '\0' is stored after the last character in the buffer.
+ * @param S A Socket_T object
+ * @param s A character buffer to store the string in
  * @param size The size of the string buffer, s
  * @return s on success, and NULL on error or when end of file occurs
  * while no characters have been read.
  */
 char *socket_readln(Socket_T S, char *s, int size) {
-
-  char *p= s;
+  
+  int c;
+  unsigned char *p= (unsigned char *)s;
   
   ASSERT(S);
-
-  /*
-   * TODO: We should use an internal buffer and read a large chunk
-   * from the socket not just one by one byte.
-   */
-  while(--size && ((socket_read(S, s, 1)) > 0)) {
-    if(*s++ == 10)
-	break;
+  
+  while(--size && ((c= socket_read_byte(S)) > 0)) {
+    *p++= c;
+    if(c == '\n' || c == '\0')
+      break;
   }
   
-  *s= 0;
+  *p= 0;
   
-  if(*p)
-      return p;
+  if(*s)
+    return s;
   
   return NULL;
   
 }
+
+
+/**
+ * Clears any data that exists in the input buffer
+ * @param S A Socket_T object
+ */
+void socket_reset(Socket_T S) {
+
+  ASSERT(S);
+  
+  /* Also throw away any pending incomming data */
+  while(fill(S, 0) > 0);
+  S->offset= 0;
+  S->length= 0;
+  
+}
+
+
+/* --------------------------------------------------------------- Private */
+
+
+/*
+ * Fill the internal buffer. If an error occurs or if the read
+ * operation timed out -1 is returned.
+ * @param S A Socket object
+ * @param timeout The number of seconds to wait for data to be read
+ * @return TRUE (the length of data read) or -1 if an error occured
+ */
+static int fill(Socket_T S, int timeout) {
+  
+  int n;
+  
+  S->offset= 0;
+  S->length= 0;
+  /* Optimizing, assuming a request/response pattern and that a udp_write
+     was issued before we are called, we don't have to wait for data */
+  if(S->type==SOCK_DGRAM) timeout= 0; 
+  
+  /* Read as much as we can, but only block on the first read */
+  while(RBUFFER_SIZE > S->length) {
+    
+    if(S->ssl) {
+      n= recv_ssl_socket(S->ssl, &S->buffer[S->length], 
+                         RBUFFER_SIZE-S->length, timeout);
+    } else {
+      n= sock_read(S->socket, &S->buffer[S->length], 
+                   RBUFFER_SIZE-S->length, timeout);
+    }
+    
+    timeout= 0;
+    
+    if(n > 0) {
+      S->length+= n;
+      continue;
+    }
+    
+    if(n==0) break;
+        
+    if(n < 0) {
+      if(errno == EAGAIN || errno == EWOULDBLOCK || S->type==SOCK_DGRAM) break;
+      return -1;
+    }
+    
+  }
+  
+  return S->length;
+  
+}
