@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -150,7 +150,7 @@ static void handle_ssl_error(char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: ssl.c,v 1.42 2005/08/05 09:38:09 chopp Exp $
+ *  @version \$Id: ssl.c,v 1.45 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -1184,9 +1184,10 @@ ssl_connection *accept_ssl_socket(ssl_server_connection *ssl_server) {
  * @param ssl ssl connection
  * @param buffer array containg the data
  * @param len size of the data container
+ * @param timeout Seconds to wait for data to be written
  * @return number of bytes transmitted, -1 in case of an error
  */
-int send_ssl_socket(ssl_connection *ssl, void *buffer, int len) {
+int send_ssl_socket(ssl_connection *ssl, void *buffer, int len, int timeout) {
 
 #ifdef HAVE_OPENSSL
   int n= 0;
@@ -1197,7 +1198,7 @@ int send_ssl_socket(ssl_connection *ssl, void *buffer, int len) {
     n= SSL_write(ssl->handler, buffer, len);
   } while(n <= 0 &&
 	  BIO_should_retry(ssl->socket_bio) &&
-	  can_write(ssl->socket, 5));
+	  can_write(ssl->socket, timeout));
   
   if(n <= 0) {
     return -1;
@@ -1231,11 +1232,11 @@ int recv_ssl_socket(ssl_connection *ssl, void *buffer, int len, int timeout) {
 
   do {
     n= SSL_read(ssl->handler, buffer, len);
-  } while(n <= 0 &&
+  } while(n < 0 &&
 	  BIO_should_retry(ssl->socket_bio) &&
 	  can_read(ssl->socket, timeout));
   
-  if(n <= 0) {
+  if(n < 0) {
     return -1;
   }
 
