@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -60,7 +60,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: status.c,v 1.57 2005/04/05 19:52:57 martinp Exp $
+ *  @version \$Id: status.c,v 1.60 2006/01/01 22:44:02 martinp Exp $
  *
  *  @file
  */
@@ -70,47 +70,50 @@
 
 
 /**
- * Show all services in the service list
+ * Show all services in the service list.
  */
-void status(char *level) {
+int status(char *level) {
 
 #define LINE 1024
 
-  if(exist_daemon()) {
-    Socket_T sock= socket_new(Run.bind_addr?Run.bind_addr:"localhost",
-			      Run.httpdport, SOCKET_TCP, Run.httpdssl,
-                              NET_TIMEOUT);
-    if(!sock) {
-      log("%s: error connecting to the monit daemon\n", prog);
-      return;
-    } else {
-      char buf[LINE];
-      int status= FALSE;
-      char *auth= Util_getBasicAuthHeader();
-      
-      socket_print(sock, 
-		   "GET /_status?format=text&level=%s HTTP/1.0\r\n%s\r\n",
-		   level, auth?auth:"");
-      FREE(auth);
-      /* Read past HTTP headers and check status */
-      while(socket_readln(sock, buf, LINE)) {
-	if(IS(buf, "\n") || IS(buf, "\r\n"))
-	  break;
-	if(Util_startsWith(buf, "HTTP/1.0 200"))
-	  status= TRUE;
-      }
-      if(!status) {
-	log("%s: cannot read status from the monit daemon\n", prog);
-	goto error;
-      }
-      while(socket_readln(sock, buf, LINE)) {
-	printf("%s", buf);
-      }
-    error:
-      socket_free(&sock);
-    }
-  } else {
+  int status= FALSE;
+  Socket_T sock = NULL;
+  char buf[LINE];
+  char *auth= NULL;
+
+  if(!exist_daemon()) {
     log("%s: no status available -- the monit daemon is not running\n", prog);
+    return status;
   }
-  
+
+  if(!(sock= socket_new(Run.bind_addr?Run.bind_addr:"localhost", Run.httpdport,
+                        SOCKET_TCP, Run.httpdssl, NET_TIMEOUT))) {
+    log("%s: error connecting to the monit daemon\n", prog);
+    return status;
+  }
+
+  auth= Util_getBasicAuthHeader();
+  socket_print(sock, 
+	       "GET /_status?format=text&level=%s HTTP/1.0\r\n%s\r\n",
+	       level, auth?auth:"");
+  FREE(auth);
+
+  /* Read past HTTP headers and check status */
+  while(socket_readln(sock, buf, LINE)) {
+    if(IS(buf, "\n") || IS(buf, "\r\n"))
+      break;
+    if(Util_startsWith(buf, "HTTP/1.0 200"))
+      status= TRUE;
+  }
+
+  if(!status) {
+    log("%s: cannot read status from the monit daemon\n", prog);
+  } else {
+    while(socket_readln(sock, buf, LINE)) {
+      printf("%s", buf);
+    }
+  }
+  socket_free(&sock);
+
+  return status;
 }
