@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -119,7 +119,7 @@ static int is_url_unsafe(unsigned char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.164 2005/09/25 21:02:54 martinp Exp $
+ *  @version \$Id: util.c,v 1.180 2006/01/01 22:44:02 martinp Exp $
  *  @file
  */
 
@@ -294,7 +294,10 @@ char *Util_replaceString(char **src, const char *old, const char *new) {
   int i;
   int d;
   
-  ASSERT(src && *src && old && new);
+  ASSERT(src);
+  ASSERT(*src);
+  ASSERT(old);
+  ASSERT(new);
   
   i= Util_countWords(*src, old);
   d= strlen(new)-strlen(old);
@@ -382,10 +385,10 @@ int Util_startsWith(const char *a, const char *b) {
  * @param buf A string
  */
 void Util_handleEscapes(char *buf) {
-
+  
   int editpos;
   int insertpos;
-
+  
   ASSERT(buf);
   
   for(editpos=insertpos=0; *(buf+editpos)!='\0'; editpos++, insertpos++) {
@@ -393,46 +396,53 @@ void Util_handleEscapes(char *buf) {
     if(*(buf+editpos) == '\\' ) {
       
       switch(*(buf+editpos+1)) {
-	
-      case 'n': 
-        *(buf+insertpos)='\n';
-        editpos++;
-	break;
-
-      case 't':
-        *(buf+insertpos)='\t';
-        editpos++;
-	break;
-
-      case 'r':
-        *(buf+insertpos)='\r';
-        editpos++;
-	break;
-
-      case ' ':
-        *(buf+insertpos)=' ';
-        editpos++;
-	break;
-
-      case '\\':
-        *(buf+insertpos)='\\';
-        editpos++;
-	break;
-
-      default:
-        *(buf+insertpos)=*(buf+editpos);
-
+        
+        case 'n': 
+          *(buf+insertpos)='\n';
+          editpos++;
+          break;
+          
+        case 't':
+          *(buf+insertpos)='\t';
+          editpos++;
+          break;
+          
+        case 'r':
+          *(buf+insertpos)='\r';
+          editpos++;
+          break;
+          
+        case ' ':
+          *(buf+insertpos)=' ';
+          editpos++;
+          break;
+          
+        case '0':
+          if(*(buf+editpos+2)=='x') {
+            *(buf+insertpos)=x2c(&buf[editpos+3]);
+            editpos+=4;
+          }
+          break;
+          
+        case '\\':
+          *(buf+insertpos)='\\';
+          editpos++;
+          break;
+          
+        default:
+          *(buf+insertpos)=*(buf+editpos);
+          
       }  
-
+      
     } else {
-
+      
       *(buf+insertpos)=*(buf+editpos);
-
+      
     }  
-
+    
   }
   *(buf+insertpos)='\0';
-
+  
 }
 
 
@@ -506,6 +516,18 @@ void Util_printRunList() {
   printf(" %-18s = %s\n", "Use process engine", Run.doprocess?"True":"False");
   printf(" %-18s = %d seconds\n", "Poll time", Run.polltime);
 
+  if(Run.eventlist_dir) {
+    char slots[STRLEN];
+
+    if(Run.eventlist_slots < 0)
+      snprintf(slots, STRLEN, "unlimited"); 
+    else
+      snprintf(slots, STRLEN, "%d", Run.eventlist_slots);
+
+    printf(" %-18s = base directory %s with %s slots\n",
+      "Event queue", Run.eventlist_dir, slots);
+  }
+
   if(Run.collectors) {
     Collector_T c= Run.collectors;
     printf(" %-18s = ", "Collector(s)");
@@ -580,10 +602,10 @@ void Util_printRunList() {
     Mail_T list;
     for(list= Run.maillist; list; list= list->next) {
       printf(" %-18s = %s\n", "Alert mail to", is_str_defined(list->to));
-      printf(" %-18s = ", "Alert on");
+      printf("   %-16s = ", "Alert on");
       printevents(list->events);
       if(list->reminder)
-        printf(" %-18s = %u cycles", "Alert reminder", list->reminder);
+        printf("   %-16s = %u cycles\n", "Alert reminder", list->reminder);
     }
   }
 
@@ -615,10 +637,11 @@ void Util_printService(Service_T s) {
  
   snprintf(string, STRLEN, "%s Name", servicetypes[s->type]);
   printf("%-21s = %s\n", string, s->name);
-  printf(" %-20s = %s\n", "Group", is_str_defined(s->group));
+  if(s->group)
+    printf(" %-20s = %s\n", "Group", s->group);
   if(s->type == TYPE_PROCESS)
     printf(" %-20s = %s\n", "Pid file", s->path);
-  else if(s->type != TYPE_HOST)
+  else if(s->type != TYPE_HOST && s->type != TYPE_SYSTEM)
     printf(" %-20s = %s\n", "Path", s->path);
   printf(" %-20s = %s\n", "Monitoring mode", modenames[s->mode]);
   if(s->start) {
@@ -731,29 +754,17 @@ void Util_printService(Service_T s) {
 
       if(n->family == AF_INET) {
 	
-	   if(n->SSL.use_ssl) {
+	   printf(" %-20s = if failed %s:%d%s [%s via %s] with "
+             "timeout %d seconds %s then %s else if passed %s then %s\n",
+	     "Port", n->hostname, n->port, n->request?n->request:"",
+             n->protocol->name, Util_portTypeDescription(n),
+             n->timeout,
+             ratio1, actionnames[a->failed->id],
+             ratio2, actionnames[a->passed->id]);
 	  
-	     printf(" %-20s = if failed %s:%d%s [protocol %s via SSL] with "
-               "timeout %d seconds %s then %s else if passed %s then %s\n",
-	       "Port", n->hostname, n->port, n->request?n->request:"",
-	       n->protocol->name, n->timeout,
-               ratio1, actionnames[a->failed->id],
-               ratio2, actionnames[a->passed->id]);
-	  
-	     if(n->SSL.certmd5 != NULL)
-	       printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
-	  
-        } else {
+	   if(n->SSL.certmd5 != NULL)
+	     printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
 
-	     printf(" %-20s = if failed %s:%d%s [protocol %s] with timeout %d "
-               "seconds %s then %s else if passed %s then %s\n",
-               "Port", n->hostname, n->port, n->request?n->request:"",
-               n->protocol->name, n->timeout,
-               ratio1, actionnames[a->failed->id],
-               ratio2, actionnames[a->passed->id]);
-	  
-	   }
-	
       } else if(n->family == AF_UNIX) {
 	
         printf(" %-20s = if failed %s [protocol %s] with timeout %d seconds "
@@ -879,6 +890,36 @@ void Util_printService(Service_T s) {
         ratio2, actionnames[a->passed->id]);
       break;
 
+    case RESOURCE_ID_CPUUSER: 
+
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if passed %s then %s\n", 
+        "CPU user limit", 
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
+      break;
+
+    case RESOURCE_ID_CPUSYSTEM: 
+
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if passed %s then %s\n", 
+        "CPU system limit", 
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
+      break;
+
+    case RESOURCE_ID_CPUWAIT: 
+
+      printf(" %-20s = if %s %.1f%% %s then %s "
+        "else if passed %s then %s\n", 
+        "CPU wait limit", 
+        operatornames[q->operator], q->limit/10.0,
+        ratio1, actionnames[a->failed->id],
+        ratio2, actionnames[a->passed->id]);
+      break;
+
     case RESOURCE_ID_MEM_PERCENT: 
 
       printf(" %-20s = if %s %.1f%% %s then %s "
@@ -974,10 +1015,10 @@ void Util_printService(Service_T s) {
 
   for(r= s->maillist; r; r= r->next) {
     printf(" %-20s = %s\n", "Alert mail to", is_str_defined(r->to));
-    printf(" %-20s = ", "Alert on");
+    printf("   %-18s = ", "Alert on");
     printevents(r->events);
     if(r->reminder)
-      printf(" %-20s = %u cycles", "Alert reminder", r->reminder);
+      printf("   %-18s = %u cycles\n", "Alert reminder", r->reminder);
   }
 
   printf("\n");
@@ -1069,16 +1110,17 @@ pid_t Util_getPid(char *pidfile) {
   ASSERT(pidfile);
 
   if(! File_exist(pidfile)) {
-    return(FALSE);
+    DEBUG("%s: pidfile '%s' does not exist\n",prog, pidfile);
+    return FALSE;
   }
   if(! File_isFile(pidfile)) {
     log("%s: pidfile '%s' is not a regular file\n",prog, pidfile);
-    return(FALSE);
+    return FALSE;
   }
   if((file= fopen(pidfile,"r")) == (FILE *)NULL) {
     log("%s: Error opening the pidfile '%s' -- %s\n",
 	prog, pidfile, STRERROR);
-    return(FALSE);
+    return FALSE ;
   }
   fscanf(file, "%d", &pid);
   fclose(file);
@@ -1100,13 +1142,15 @@ pid_t Util_getPid(char *pidfile) {
 int Util_isProcessRunning(Service_T s) {
   
   pid_t  pid;
-
+  
   ASSERT(s);
   
   errno= 0;
-  if((pid= Util_getPid(s->path)))
+  if((pid= Util_getPid(s->path)) >= 0) {
     if( (getpgid(pid) > -1) || (errno == EPERM) )
       return pid;
+    DEBUG("'%s' Error testing process id [%d] -- %s\n", s->name, pid, STRERROR);
+	}
   Util_resetInfo(s);
   
   return FALSE;
@@ -1123,7 +1167,7 @@ int Util_isProcessRunning(Service_T s) {
  * @param len the length of the result buffer
  * @return a pointer to the result buffer
  */
-char *Util_getRFC822Date(long *date, char *result, int len) {
+char *Util_getRFC822Date(time_t *date, char *result, int len) {
   
   struct tm *tm_now;
   time_t now= (date && *date>0)?*date:time(NULL);
@@ -1763,6 +1807,23 @@ char *Util_getEventratio(Action_T action, char *buf) {
 }
 
 
+/**
+ * Print port type description
+ * @param p A port structure
+ * @return the socket type description
+ */
+char *Util_portTypeDescription(Port_T p) {
+  switch(p->type) {
+    case SOCK_STREAM:
+      return p->SSL.use_ssl?"TCPSSL":"TCP";
+    case SOCK_DGRAM:
+      return "UDP";
+    default:
+      return "UNKNOWN";
+  }
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -1831,6 +1892,8 @@ static void printevents(unsigned int events) {
       printf("Gid ");
     if(IS_EVENT_SET(events, EVENT_ICMP))
       printf("Icmp ");
+    if(IS_EVENT_SET(events, EVENT_INSTANCE))
+      printf("Instance ");
     if(IS_EVENT_SET(events, EVENT_INVALID))
       printf("Invalid ");
     if(IS_EVENT_SET(events, EVENT_MATCH))
