@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.h,v 1.11 2005/09/05 09:51:38 martinp Exp $
+ *  @version \$Id: util.h,v 1.14 2006/01/01 22:44:02 martinp Exp $
  *  @file
  */
 
@@ -209,7 +209,7 @@ int Util_isProcessRunning(Service_T s);
  * @param len the length of the result buffer
  * @return a pointer to the result buffer
  */
-char *Util_getRFC822Date(long *date, char *result, int len);
+char *Util_getRFC822Date(time_t *date, char *result, int len);
 
 
 /**
@@ -395,4 +395,12 @@ int Util_getAction(const char *action);
 char *Util_getEventratio(Action_T action, char *buf);
 
 
+/**
+ * Print port type description
+ * @param p A port structure
+ * @return the socket type description
+ */
+char *Util_portTypeDescription(Port_T p);
+
+
 #endif
