@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2005 by the monit project group.
+ * Copyright (C), 2000-2006 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -90,7 +90,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: validate.c,v 1.145 2005/09/30 10:25:14 martinp Exp $
+ *  @version \$Id: validate.c,v 1.151 2006/01/01 22:44:02 martinp Exp $
  *  @file
  */
 
@@ -102,9 +102,9 @@ static void check_uid(Service_T);
 static void check_gid(Service_T);
 static void check_size(Service_T);
 static void check_perm(Service_T);
- void check_match(Service_T);
- int  check_match_ignore(Service_T, char *);
- void check_match_if(Service_T, char *);
+static void check_match(Service_T);
+static int  check_match_ignore(Service_T, char *);
+static void check_match_if(Service_T, char *);
 static int  check_skip(Service_T);
 static int  check_timeout(Service_T);
 static void check_checksum(Service_T);
@@ -136,6 +136,9 @@ void validate() {
   Service_T s;
   sigset_t ns, os;
 
+  Run.handler_flag = HANDLER_PASSED;
+  Event_queue_process(Run.eventlist);
+
   if(Run.doprocess) {
     initprocesstree(&ptree, &ptreesize, &oldptree, &oldptreesize);
     update_system_load(ptree, ptreesize);
@@ -389,6 +392,57 @@ int check_directory(Service_T s) {
 }
 
 
+/**
+ * Validate a given fifo service s. Events are posted according to 
+ * its configuration. In case of a fatal event FALSE is returned.
+ */
+int check_fifo(Service_T s) {
+
+  struct stat stat_buf;
+
+  ASSERT(s);
+
+  if(stat(s->path, &stat_buf) != 0) {
+    Event_post(s, EVENT_NONEXIST, STATE_FAILED, s->action_NONEXIST,
+      "'%s' fifo doesn't exist", s->name);
+    return FALSE;
+  } else {
+    s->inf->st_mode= stat_buf.st_mode;
+    s->inf->st_uid= stat_buf.st_uid;
+    s->inf->st_gid= stat_buf.st_gid;
+    s->inf->timestamp= MAX(stat_buf.st_mtime, stat_buf.st_ctime);
+    DEBUG("'%s' fifo existence check passed\n", s->name);
+    Event_post(s, EVENT_NONEXIST, STATE_PASSED, s->action_NONEXIST,
+      "'%s' fifo exist", s->name);
+  }
+
+  if(!S_ISFIFO(s->inf->st_mode)) {
+    Event_post(s, EVENT_INVALID, STATE_FAILED, s->action_INVALID,
+      "'%s' is not fifo", s->name);
+    return FALSE;
+  } else {
+    DEBUG("'%s' is fifo\n", s->name);
+    Event_post(s, EVENT_INVALID, STATE_PASSED, s->action_INVALID,
+      "'%s' is fifo", s->name);
+  }
+
+  if(s->perm)
+    check_perm(s);
+
+  if(s->uid)
+    check_uid(s);
+
+  if(s->gid)
+    check_gid(s);
+
+  if(s->timestamplist)
+    check_timestamp(s);
+
+  return TRUE;
+
+}
+
+
 /**
  * Validate a remote service.
  * @param s The remote service to validate
@@ -452,6 +506,24 @@ int check_remote_host(Service_T s) {
 }
 
 
+/**
+ * Validate the general system indicators. In case of a fatal event
+ * FALSE is returned.
+ */
+int check_system(Service_T s) {
+
+  Resource_T r= NULL;
+
+  ASSERT(s);
+
+  for(r= s->resourcelist; r; r= r->next) {
+    check_process_resources(s, r);
+  }
+
+  return TRUE;
+}
+
+
 /* --------------------------------------------------------------- Private */
 
 
@@ -477,20 +549,20 @@ static void check_connection(Service_T s, Port_T p) {
   socket= socket_create(p);
   if(!socket) {
     snprintf(report, STRLEN,
-             "'%s' failed, cannot open a connection to %s",
-             s->name, p->address);
+      "'%s' failed, cannot open a connection to %s via %s",
+      s->name, p->address, Util_portTypeDescription(p));
     rv= FALSE;
     goto error;
   } else {
-    DEBUG("'%s' succeeded connecting to %s\n",
-      s->name, p->address);
+    DEBUG("'%s' succeeded connecting to %s via %s\n",
+      s->name, p->address, Util_portTypeDescription(p));
   }
 
   /* Verify that the socket is ready for i|o */
   if(! socket_is_ready(socket)) {
     snprintf(report, STRLEN,
-             "'%s' failed, the socket at %s is not ready for i|o -- %s",
-             s->name, p->address, STRERROR);
+      "'%s' failed, the socket at %s via %s is not ready for i|o -- %s",
+      s->name, p->address, Util_portTypeDescription(p), STRERROR);
     rv= FALSE;
     goto error;
   }
@@ -498,13 +570,13 @@ static void check_connection(Service_T s, Port_T p) {
   /* Run the protocol verification routine through the socket */
   if(! p->protocol->check(socket)) {
     snprintf(report, STRLEN,
-             "'%s' failed protocol test [%s] at %s.",
-             s->name, p->protocol->name, p->address);
+      "'%s' failed protocol test [%s] at %s via %s",
+      s->name, p->protocol->name, p->address, Util_portTypeDescription(p));
     rv= FALSE;
     goto error;
   } else {
-    DEBUG("'%s' succeeded testing protocol [%s] at %s\n",
-          s->name, p->protocol->name, p->address);
+    DEBUG("'%s' succeeded testing protocol [%s] at %s via %s\n",
+      s->name, p->protocol->name, p->address, Util_portTypeDescription(p));
   }
 
   /* Get time of connection attempt finish */
@@ -523,7 +595,8 @@ static void check_connection(Service_T s, Port_T p) {
   } else {
     p->is_available= TRUE;
     Event_post(s, EVENT_CONNECTION, STATE_PASSED, p->action,
-      "'%s' connection passed", s->name);
+      "'%s' connection passed to %s via %s", s->name,
+      p->address, Util_portTypeDescription(p));
   }
       
 }
@@ -599,20 +672,21 @@ static void check_process_ppid(Service_T s) {
 /**
  * Check process resources
  */
-static void check_process_resources(Service_T s, Resource_T pr) {
+static void check_process_resources(Service_T s, Resource_T r) {
 
   int okay= TRUE;
   char report[STRLEN]={0};
 
-  ASSERT(s && pr);
+  ASSERT(s && r);
+
+  switch(r->resource_id) {
 
-  switch(pr->resource_id) {
   case RESOURCE_ID_CPU_PERCENT:
-    if(Util_evalQExpression(pr->operator, s->inf->cpu_percent, pr->limit)) {
+    if(Util_evalQExpression(r->operator, s->inf->cpu_percent, r->limit)) {
       snprintf(report, STRLEN,
         "'%s' cpu usage of %.1f%% matches resource limit [cpu usage%s%.1f%%]",
-        s->name, s->inf->cpu_percent/10.0, operatorshortnames[pr->operator],
-        pr->limit/10.0);
+        s->name, s->inf->cpu_percent/10.0, operatorshortnames[r->operator],
+        r->limit/10.0);
       okay= FALSE;
     } else {
       DEBUG("'%s' cpu usage check passed [current cpu usage=%.1f%%]\n",
@@ -620,39 +694,103 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     }
     break;
 
-  case RESOURCE_ID_MEM_PERCENT:
-    if(Util_evalQExpression(pr->operator, s->inf->mem_percent, pr->limit)) {
+  case RESOURCE_ID_CPUUSER:
+    if(Util_evalQExpression(r->operator, systeminfo.total_cpu_user_percent, r->limit)) {
       snprintf(report, STRLEN,
-        "'%s' mem usage of %.1f%% matches resource limit [mem usage%s%.1f%%]",
-        s->name, s->inf->mem_percent/10.0, operatorshortnames[pr->operator],
-        pr->limit/10.0);
+        "'%s' cpu user usage of %.1f%% matches resource limit [cpu user usage%s%.1f%%]",
+        s->name, systeminfo.total_cpu_user_percent/10.0, operatorshortnames[r->operator],
+        r->limit/10.0);
       okay= FALSE;
     } else {
-      DEBUG("'%s' mem usage check passed [current mem usage=%.1f%%]\n",
-        s->name, s->inf->mem_percent/10.0);
+      DEBUG("'%s' cpu user usage check passed [current cpu user usage=%.1f%%]\n",
+        s->name, systeminfo.total_cpu_user_percent/10.0);
+    }
+    break;
+
+  case RESOURCE_ID_CPUSYSTEM:
+    if(Util_evalQExpression(r->operator, systeminfo.total_cpu_syst_percent, r->limit)) {
+      snprintf(report, STRLEN,
+        "'%s' cpu system usage of %.1f%% matches resource limit [cpu system usage%s%.1f%%]",
+        s->name, systeminfo.total_cpu_syst_percent/10.0, operatorshortnames[r->operator],
+        r->limit/10.0);
+      okay= FALSE;
+    } else {
+      DEBUG("'%s' cpu system usage check passed [current cpu system usage=%.1f%%]\n",
+        s->name, systeminfo.total_cpu_syst_percent/10.0);
+    }
+    break;
+
+  case RESOURCE_ID_CPUWAIT:
+    if(Util_evalQExpression(r->operator, systeminfo.total_cpu_wait_percent, r->limit)) {
+      snprintf(report, STRLEN,
+        "'%s' cpu wait usage of %.1f%% matches resource limit [cpu wait usage%s%.1f%%]",
+        s->name, systeminfo.total_cpu_wait_percent/10.0, operatorshortnames[r->operator],
+        r->limit/10.0);
+      okay= FALSE;
+    } else {
+      DEBUG("'%s' cpu wait usage check passed [current cpu wait usage=%.1f%%]\n",
+        s->name, systeminfo.total_cpu_wait_percent/10.0);
+    }
+    break;
+
+  case RESOURCE_ID_MEM_PERCENT:
+    if(s->type == TYPE_SYSTEM) {
+      if(Util_evalQExpression(r->operator, systeminfo.total_mem_percent, r->limit)) {
+        snprintf(report, STRLEN,
+          "'%s' mem usage of %.1f%% matches resource limit [mem usage%s%.1f%%]",
+          s->name, systeminfo.total_mem_percent/10.0, operatorshortnames[r->operator],
+          r->limit/10.0);
+        okay= FALSE;
+      } else {
+        DEBUG("'%s' mem usage check passed [current mem usage=%.1f%%]\n",
+          s->name, systeminfo.total_mem_percent/10.0);
+      }
+    } else {
+      if(Util_evalQExpression(r->operator, s->inf->mem_percent, r->limit)) {
+        snprintf(report, STRLEN,
+          "'%s' mem usage of %.1f%% matches resource limit [mem usage%s%.1f%%]",
+          s->name, s->inf->mem_percent/10.0, operatorshortnames[r->operator],
+          r->limit/10.0);
+        okay= FALSE;
+      } else {
+        DEBUG("'%s' mem usage check passed [current mem usage=%.1f%%]\n",
+          s->name, s->inf->mem_percent/10.0);
+      }
     }
     break;
 
   case RESOURCE_ID_MEM_KBYTE:
-    if(Util_evalQExpression(pr->operator, s->inf->mem_kbyte, pr->limit)) {
-      snprintf(report, STRLEN,
-        "'%s' mem amount of %ldkB matches resource limit [mem amount%s%ldkB]",
-        s->name, s->inf->mem_kbyte, operatorshortnames[pr->operator], pr->limit);
-      okay= FALSE;
+    if(s->type == TYPE_SYSTEM) {
+      if(Util_evalQExpression(r->operator, systeminfo.total_mem_kbyte, r->limit)) {
+        snprintf(report, STRLEN,
+          "'%s' mem amount of %ldkB matches resource limit [mem amount%s%ldkB]",
+          s->name, systeminfo.total_mem_kbyte, operatorshortnames[r->operator], r->limit);
+        okay= FALSE;
+      } else {
+        DEBUG("'%s' mem amount check passed [current mem amount=%ldkB]\n",
+          s->name, systeminfo.total_mem_kbyte);
+      }
     } else {
-      DEBUG("'%s' mem amount check passed [current mem amount=%ldkB]\n",
-        s->name, s->inf->mem_kbyte);
+      if(Util_evalQExpression(r->operator, s->inf->mem_kbyte, r->limit)) {
+        snprintf(report, STRLEN,
+          "'%s' mem amount of %ldkB matches resource limit [mem amount%s%ldkB]",
+          s->name, s->inf->mem_kbyte, operatorshortnames[r->operator], r->limit);
+        okay= FALSE;
+      } else {
+        DEBUG("'%s' mem amount check passed [current mem amount=%ldkB]\n",
+          s->name, s->inf->mem_kbyte);
+      }
     }
     break;
 
   case RESOURCE_ID_LOAD1:
-    if(Util_evalQExpression(pr->operator,
-                     (int)(systeminfo.loadavg[0]*10.0), pr->limit)) {
+    if(Util_evalQExpression(r->operator,
+                     (int)(systeminfo.loadavg[0]*10.0), r->limit)) {
       snprintf(report, STRLEN,
         "'%s' loadavg(1min) of %.1f matches resource limit "
         "[loadavg(1min)%s%.1f]",
-        s->name, systeminfo.loadavg[0], operatorshortnames[pr->operator],
-        pr->limit/10.0);
+        s->name, systeminfo.loadavg[0], operatorshortnames[r->operator],
+        r->limit/10.0);
       okay= FALSE;
     } else {
       DEBUG("'%s' loadavg(1min) check passed [current loadavg(1min)=%.1f]\n",
@@ -661,13 +799,13 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   case RESOURCE_ID_LOAD5:
-    if(Util_evalQExpression(pr->operator,
-                     (int)(systeminfo.loadavg[1]*10.0), pr->limit)) {
+    if(Util_evalQExpression(r->operator,
+                     (int)(systeminfo.loadavg[1]*10.0), r->limit)) {
       snprintf(report, STRLEN,
         "'%s' loadavg(5min) of %.1f matches resource limit "
         "[loadavg(5min)%s%.1f]",
-        s->name, systeminfo.loadavg[1], operatorshortnames[pr->operator],
-        pr->limit/10.0);
+        s->name, systeminfo.loadavg[1], operatorshortnames[r->operator],
+        r->limit/10.0);
       okay= FALSE;
     } else {
       DEBUG("'%s' loadavg(5min) check passed [current loadavg(5min)=%.1f]\n",
@@ -676,13 +814,13 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   case RESOURCE_ID_LOAD15:
-    if(Util_evalQExpression(pr->operator,
-                     (int)(systeminfo.loadavg[2]*10.0), pr->limit)) {
+    if(Util_evalQExpression(r->operator,
+                     (int)(systeminfo.loadavg[2]*10.0), r->limit)) {
       snprintf(report, STRLEN,
         "'%s' loadavg(15min) of %.1f matches resource limit "
         "[loadavg(15min)%s%.1f]",
-        s->name, systeminfo.loadavg[2], operatorshortnames[pr->operator],
-        pr->limit/10.0);
+        s->name, systeminfo.loadavg[2], operatorshortnames[r->operator],
+        r->limit/10.0);
       okay= FALSE;
     } else {
       DEBUG("'%s' loadavg(15min) check passed [current loadavg(15min)=%.1f]\n",
@@ -691,10 +829,10 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   case RESOURCE_ID_CHILDREN:
-    if(Util_evalQExpression(pr->operator, s->inf->children, pr->limit)) {
+    if(Util_evalQExpression(r->operator, s->inf->children, r->limit)) {
       snprintf(report, STRLEN,
         "'%s' children of %i matches resource limit [children%s%ld]",
-        s->name, s->inf->children, operatorshortnames[pr->operator], pr->limit);
+        s->name, s->inf->children, operatorshortnames[r->operator], r->limit);
       okay= FALSE;
     } else {
       DEBUG("'%s' children check passed [current children=%i]\n",
@@ -703,12 +841,12 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   case RESOURCE_ID_TOTAL_MEM_KBYTE:
-    if(Util_evalQExpression(pr->operator, s->inf->total_mem_kbyte, pr->limit)) {
+    if(Util_evalQExpression(r->operator, s->inf->total_mem_kbyte, r->limit)) {
       snprintf(report, STRLEN,
         "'%s' total mem amount of %ldkB matches resource limit"
         " [total mem amount%s%ldkB]",
-        s->name, s->inf->total_mem_kbyte, operatorshortnames[pr->operator],
-        pr->limit);
+        s->name, s->inf->total_mem_kbyte, operatorshortnames[r->operator],
+        r->limit);
       okay= FALSE;
     } else {
       DEBUG("'%s' total mem amount check passed "
@@ -717,12 +855,12 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   case RESOURCE_ID_TOTAL_MEM_PERCENT:
-    if(Util_evalQExpression(pr->operator, s->inf->total_mem_percent, pr->limit)) {
+    if(Util_evalQExpression(r->operator, s->inf->total_mem_percent, r->limit)) {
       snprintf(report, STRLEN,
         "'%s' total mem amount of %.1f%% matches resource limit"
         " [total mem amount%s%.1f%%]",
         s->name, (float)s->inf->total_mem_percent/10.0,
-        operatorshortnames[pr->operator], (float)pr->limit/10.0);
+        operatorshortnames[r->operator], (float)r->limit/10.0);
       okay= FALSE;
     } else {
       DEBUG("'%s' total mem amount check passed "
@@ -732,14 +870,14 @@ static void check_process_resources(Service_T s, Resource_T pr) {
     break;
 
   default:
-    log("'%s' error -- unknown resource ID: [%d]\n", s->name, pr->resource_id);
+    log("'%s' error -- unknown resource ID: [%d]\n", s->name, r->resource_id);
     return;
   }
 
   if(! okay) {
-    Event_post(s, EVENT_RESOURCE, STATE_FAILED, pr->action, report);
+    Event_post(s, EVENT_RESOURCE, STATE_FAILED, r->action, "%s", report);
   } else {
-    Event_post(s, EVENT_RESOURCE, STATE_PASSED, pr->action,
+    Event_post(s, EVENT_RESOURCE, STATE_PASSED, r->action,
       "'%s' resource passed", s->name);
   }
   
@@ -888,7 +1026,6 @@ static void check_timestamp(Service_T s) {
 
   ASSERT(s && s->timestamplist);
 
-
   if((int)time(&now) == -1) {
     Event_post(s, EVENT_DATA, STATE_FAILED, s->action_DATA,
       "'%s' can't obtain actual system time", s->name);
@@ -900,12 +1037,14 @@ static void check_timestamp(Service_T s) {
 
   for(t= s->timestamplist; t; t= t->next) {
     if(t->test_changes) {
-    /* if we are testing for changes only, the value is variable */
+
+      /* if we are testing for changes only, the value is variable */
+
       if(t->timestamp != s->inf->timestamp) {
+        t->timestamp= s->inf->timestamp;
         Event_post(s, EVENT_CHANGED, STATE_FAILED, t->action,
           "'%s' timestamp was changed for %s", s->name, s->path);
         /* reset expected value for next cycle */
-        t->timestamp= s->inf->timestamp;
       } else {
         DEBUG("'%s' timestamp was not changed for %s\n", s->name, s->path);
         Event_post(s, EVENT_CHANGED, STATE_PASSED, t->action,
@@ -913,7 +1052,9 @@ static void check_timestamp(Service_T s) {
       }
       break;
     } else {
-    /* we are testing constant value for failed or passed state */
+
+      /* we are testing constant value for failed or passed state */
+
       if(Util_evalQExpression(t->operator, (int)(now - s->inf->timestamp), t->time)) {
         Event_post(s, EVENT_TIMESTAMP, STATE_FAILED, t->action,
           "'%s' timestamp test failed for %s", s->name, s->path);
@@ -970,7 +1111,7 @@ static void check_size(Service_T s) {
 /**
  * Match content
  */
- void check_match(Service_T s) {
+static void check_match(Service_T s) {
   char    line[MATCH_LINE_LENGTH];
   FILE    *file;
   int     inode_checked=FALSE;
@@ -1073,7 +1214,7 @@ static void check_size(Service_T s) {
 /**
  * Match line for "ignore" statements
  */
- int check_match_ignore(Service_T s, char *line) {
+static int check_match_ignore(Service_T s, char *line) {
 
   int     rv=FALSE;
   Match_T ml;
@@ -1123,7 +1264,7 @@ static void check_size(Service_T s) {
 /**
  * Match line for "if" statements
  */
- void check_match_if(Service_T s, char *line) {
+static void check_match_if(Service_T s, char *line) {
 
   Match_T ml;
   int     match_return;
